@@ -240,6 +240,7 @@ typedef enum
 typedef struct DscCaps
 {
     NvBool bDSCSupported;
+    NvBool bDSCDecompressionSupported;
     NvBool bDSCPassThroughSupported;
     unsigned versionMajor, versionMinor;
     unsigned rcBufferBlockSize;
@@ -289,6 +290,7 @@ typedef struct
 {
     NvBool  bSourceControlModeSupported;
     NvBool  bConcurrentLTSupported;
+    NvBool  bConv444To420Supported;
     NvU8    maxTmdsClkRate;
     NvU8    maxBpc;
     NvU8    maxHdmiLinkBandwidthGbps;
@@ -458,6 +460,20 @@ typedef struct PanelReplayConfig
     NvBool   enablePanelReplay;
 } panelReplayConfig;
 
+// PR state
+typedef enum
+{
+    PanelReplay_Inactive            = 0,
+    PanelReplay_CaptureAndDisplay   = 1,
+    PanelReplay_DisplayFromRfb      = 2,
+    PanelReplay_Undefined           = 7
+} PanelReplayState;
+
+typedef struct
+{
+    PanelReplayState prState;
+} PanelReplayStatus;
+
 // Multiplier constant to get link frequency in KHZ
 // Maximum link rate of Main Link lanes = Value x 270M.
 // To get it to KHz unit, we need to multiply 270K.
