@@ -34,14 +34,22 @@ NvlStatus
 nvlink_core_train_check_link_ready_ALI
 (
     nvlink_link **links,
-    NvU32 linkCount
+    NvU32         linkCount
 )
 {
-    NvU32 i = 0;
+    NvU32     i      = 0;
     NvlStatus status = NVL_SUCCESS;
 
+    if (links == NULL)
+    {
+        return NVL_BAD_ARGS;
+    }
+
     for (i = 0; i < linkCount; i++)
     {
+        if (links[i] == NULL)
+            continue;
+
         if (!nvlink_core_check_link_state(links[i], NVLINK_LINKSTATE_ALI))
         {
             // If link is not in active, update status to be error and continue
@@ -89,6 +97,9 @@ nvlink_core_train_internode_conns_from_swcfg_to_active
 
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         // Don't do anything if the link is already at HS.
         if ((nvlink_core_check_link_state(conns[i]->local_end, NVLINK_LINKSTATE_HS)) &&
             (nvlink_core_check_tx_sublink_state(conns[i]->local_end,
@@ -133,7 +144,7 @@ nvlink_core_train_internode_conns_from_swcfg_to_active
 
     for (i = 0; i < connCount; i++)
     {
-        if (skipConn[i])
+        if ((conns[i] == NULL) || skipConn[i])
         {
             continue;
         }
@@ -151,6 +162,8 @@ nvlink_core_train_internode_conns_from_swcfg_to_active
 
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
 
         // Wait for the link state to change.
         status = nvlink_core_poll_link_state(conns[i]->local_end,
@@ -202,6 +215,11 @@ nvlink_core_train_internode_conn_sublink_from_safe_to_hs
 {
     NvlStatus status = NVL_SUCCESS;
 
+    if (conn == NULL)
+    {
+        return NVL_BAD_ARGS;
+    }
+
     // NVLink 3.0 onwards this is handled through INITOPTIMIZE, return error
     if (conn->local_end->version >= NVLINK_DEVICE_VERSION_30)
     {
@@ -328,6 +346,9 @@ nvlink_core_train_intranode_conns_from_from_L2_to_active
     // STEP 1: Reset all endpoints of the links. This clears any link state
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         conns[i]->end0->link_handlers->set_dl_link_mode(conns[i]->end0,
                                                         NVLINK_LINKSTATE_RESET,
                                                         flags);
@@ -339,6 +360,9 @@ nvlink_core_train_intranode_conns_from_from_L2_to_active
     // STEP 2: NVLink 3 and beyond, we also need to perform INITPHASE1
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         conns[i]->end0->link_handlers->set_dl_link_mode(conns[i]->end0,
                                                         NVLINK_LINKSTATE_INITPHASE1,
                                                         flags);
@@ -355,6 +379,9 @@ nvlink_core_train_intranode_conns_from_from_L2_to_active
     {
         for (i = 0; i < connCount; i++)
         {
+            if (conns[i] == NULL)
+                continue;
+
             status = conns[i]->end0->link_handlers->get_dl_link_mode(conns[i]->end0, &linkMode);
             if ((status != NVL_SUCCESS) ||
                 (linkMode == NVLINK_LINKSTATE_FAIL) || (linkMode == NVLINK_LINKSTATE_FAULT))
@@ -374,6 +401,9 @@ nvlink_core_train_intranode_conns_from_from_L2_to_active
     // Verify that all the endpoints are now in INIT state
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         status = nvlink_core_check_intranode_conn_state(conns[i], NVLINK_LINKSTATE_OFF);
         if (status != NVL_SUCCESS)
         {
@@ -389,6 +419,9 @@ nvlink_core_train_intranode_conns_from_from_L2_to_active
     // STEP 3: Restore all end point state saved while entering SLEEP state
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         if (conns[i]->end0->bStateSaved)
         {
             conns[i]->end0->link_handlers->set_dl_link_mode(conns[i]->end0,
@@ -409,6 +442,9 @@ nvlink_core_train_intranode_conns_from_from_L2_to_active
     {
         for (i = 0; i < connCount; i++)
         {
+            if (conns[i] == NULL)
+                continue;
+
             status = conns[i]->end0->link_handlers->get_dl_link_mode(conns[i]->end0, &linkMode);
             if ((status != NVL_SUCCESS) ||
                 (linkMode == NVLINK_LINKSTATE_FAIL) || (linkMode == NVLINK_LINKSTATE_FAULT))
@@ -428,6 +464,9 @@ nvlink_core_train_intranode_conns_from_from_L2_to_active
     // STEP 4: Initialize RX Termination on all end points
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         conns[i]->end0->link_handlers->set_rx_mode(conns[i]->end0,
                                                    NVLINK_SUBLINK_STATE_RX_INIT_TERM,
                                                    flags);
@@ -441,6 +480,9 @@ nvlink_core_train_intranode_conns_from_from_L2_to_active
     {
         for (i = 0; i < connCount; i++)
         {
+            if (conns[i] == NULL)
+                continue;
+
             status = conns[i]->end0->link_handlers->get_dl_link_mode(conns[i]->end0, &linkMode);
             if ((status != NVL_SUCCESS) ||
                 (linkMode == NVLINK_LINKSTATE_FAIL) || (linkMode == NVLINK_LINKSTATE_FAULT))
@@ -460,6 +502,9 @@ nvlink_core_train_intranode_conns_from_from_L2_to_active
     // STEP 5: Enable Common mode on Tx's of all endpoints
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         if (!((conns[i]->end0->tx_sublink_state == NVLINK_SUBLINK_STATE_TX_COMMON_MODE) ||
               (conns[i]->end0->tx_sublink_state == NVLINK_SUBLINK_STATE_TX_COMMON_MODE_DISABLE) ||
               (conns[i]->end0->tx_sublink_state == NVLINK_SUBLINK_STATE_TX_DATA_READY)))
@@ -483,6 +528,9 @@ nvlink_core_train_intranode_conns_from_from_L2_to_active
     {
         for (i = 0; i < connCount; i++)
         {
+            if (conns[i] == NULL)
+                continue;
+
             status = conns[i]->end0->link_handlers->get_dl_link_mode(conns[i]->end0, &linkMode);
             if ((status != NVL_SUCCESS) ||
                 (linkMode == NVLINK_LINKSTATE_FAIL) || (linkMode == NVLINK_LINKSTATE_FAULT))
@@ -502,6 +550,9 @@ nvlink_core_train_intranode_conns_from_from_L2_to_active
     // STEP 6: Put all Rx's in RXCAL
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         if (conns[i]->end0->rx_sublink_state != NVLINK_SUBLINK_STATE_RX_RXCAL)
         {
             conns[i]->end0->link_handlers->set_rx_mode(conns[i]->end0,
@@ -519,6 +570,9 @@ nvlink_core_train_intranode_conns_from_from_L2_to_active
     // STEP 7: Disable Tx common mode
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         if (!((conns[i]->end0->tx_sublink_state == NVLINK_SUBLINK_STATE_TX_COMMON_MODE_DISABLE) ||
               (conns[i]->end0->tx_sublink_state == NVLINK_SUBLINK_STATE_TX_DATA_READY)))
         {
@@ -538,6 +592,9 @@ nvlink_core_train_intranode_conns_from_from_L2_to_active
     // STEP 8: Set Data Ready and Enable
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         if (conns[i]->end0->tx_sublink_state != NVLINK_SUBLINK_STATE_TX_DATA_READY)
         {
             conns[i]->end0->link_handlers->set_tx_mode(conns[i]->end0,
@@ -557,6 +614,9 @@ nvlink_core_train_intranode_conns_from_from_L2_to_active
     {
         for (i = 0; i < connCount; i++)
         {
+            if (conns[i] == NULL)
+                continue;
+
             status = conns[i]->end0->link_handlers->get_dl_link_mode(conns[i]->end0, &linkMode);
             if ((status != NVL_SUCCESS) ||
                 (linkMode == NVLINK_LINKSTATE_FAIL) || (linkMode == NVLINK_LINKSTATE_FAULT))
@@ -576,6 +636,9 @@ nvlink_core_train_intranode_conns_from_from_L2_to_active
     // STEP 9: Set link mode to SAFE
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         conns[i]->end0->link_handlers->set_dl_link_mode(conns[i]->end0,
                                                         NVLINK_LINKSTATE_SAFE,
                                                         flags);
@@ -590,6 +653,9 @@ nvlink_core_train_intranode_conns_from_from_L2_to_active
     // Verify all the endpoints link state now reflect SAFE state
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         status = nvlink_core_poll_link_state(conns[i]->end0,
                                              NVLINK_LINKSTATE_SAFE,
                                              NVLINK_TRANSITION_SAFE_TIMEOUT);
@@ -634,6 +700,9 @@ nvlink_core_train_intranode_conns_from_from_L2_to_active
     // STEP 9: Set INITNEOGOTIATE
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         conns[i]->end0->link_handlers->set_dl_link_mode(conns[i]->end0,
                                                         NVLINK_LINKSTATE_INITNEGOTIATE,
                                                         flags);
@@ -651,6 +720,9 @@ nvlink_core_train_intranode_conns_from_from_L2_to_active
     // STEP 8: Set POST_INITNEGOTIATE
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         conns[i]->end0->link_handlers->set_dl_link_mode(conns[i]->end0,
                                                         NVLINK_LINKSTATE_POST_INITNEGOTIATE,
                                                         flags);
@@ -684,6 +756,9 @@ nvlink_core_train_intranode_conns_from_from_L2_to_active
 
         for (i = 0; i < connCount; i++)
         {
+            if (conns[i] == NULL)
+                continue;
+
             // Update the power state transition status of the link
             conns[i]->end0->powerStateTransitionStatus = nvlink_power_state_in_L0;
             conns[i]->end1->powerStateTransitionStatus = nvlink_power_state_in_L0;
@@ -729,6 +804,9 @@ nvlink_core_train_intranode_conns_from_swcfg_to_active_non_ALI
     // Trigger INITOPTIMIZE on both ends of the connection
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         conns[i]->end0->link_handlers->set_dl_link_mode(conns[i]->end0,
                                                         NVLINK_LINKSTATE_INITOPTIMIZE,
                                                         flags);
@@ -745,6 +823,9 @@ nvlink_core_train_intranode_conns_from_swcfg_to_active_non_ALI
     // Trigger POST_INITOPTIMIZE (Checks INITOPTIMIZE was successful) on both ends of the connection
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         conns[i]->end0->link_handlers->set_dl_link_mode(conns[i]->end0,
                                                         NVLINK_LINKSTATE_POST_INITOPTIMIZE,
                                                         flags);
@@ -761,6 +842,9 @@ nvlink_core_train_intranode_conns_from_swcfg_to_active_non_ALI
     // Set link modes to ACTIVE
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         // Some settings required before moving to ACTIVE
         _nvlink_core_set_link_pre_active_settings(conns[i]->end0, flags);
         _nvlink_core_set_link_pre_active_settings(conns[i]->end1, flags);
@@ -783,6 +867,8 @@ nvlink_core_train_intranode_conns_from_swcfg_to_active_non_ALI
     // Verify link mode HS on the endpoints
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
 
         pollStatus = nvlink_core_poll_link_state(conns[i]->end0,
                                                 NVLINK_LINKSTATE_HS,
@@ -881,6 +967,9 @@ nvlink_core_train_intranode_conns_from_swcfg_to_active_ALT
 
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         status = conns[i]->end0->link_handlers->get_dl_link_mode(conns[i]->end0, &linkMode);
         if (status != NVL_SUCCESS)
         {
@@ -903,7 +992,7 @@ nvlink_core_train_intranode_conns_from_swcfg_to_active_ALT
     // Trigger INITOPTIMIZE on both ends of the connection
     for (i = 0; i < connCount; i++)
     {
-        if (skipConn[i])
+        if ((conns[i] == NULL) || skipConn[i])
         {
             continue;
         }
@@ -924,7 +1013,7 @@ nvlink_core_train_intranode_conns_from_swcfg_to_active_ALT
     // Trigger POST_INITOPTIMIZE (Checks INITOPTIMIZE was successful) on both ends of the connection
     for (i = 0; i < connCount; i++)
     {
-        if (skipConn[i])
+        if ((conns[i] == NULL) || skipConn[i])
         {
             continue;
         }
@@ -945,7 +1034,7 @@ nvlink_core_train_intranode_conns_from_swcfg_to_active_ALT
     // Set link modes to ACTIVE
     for (i = 0; i < connCount; i++)
     {
-        if (skipConn[i])
+        if ((conns[i] == NULL) || skipConn[i])
         {
             continue;
         }
@@ -966,7 +1055,7 @@ nvlink_core_train_intranode_conns_from_swcfg_to_active_ALT
     // Verify link mode HS on the endpoints
     for (i = 0; i < connCount; i++)
     {
-        if (skipConn[i])
+        if ((conns[i] == NULL) || skipConn[i])
         {
             continue;
         }
@@ -1067,6 +1156,9 @@ nvlink_core_train_intranode_conns_from_swcfg_to_active_legacy
     // Enable PRBS generator on both ends of the link
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         _nvlink_core_set_sublink_pre_hs_settings(conns[i]->end0, flags);
         _nvlink_core_set_sublink_pre_hs_settings(conns[i]->end1, flags);
     }
@@ -1074,6 +1166,9 @@ nvlink_core_train_intranode_conns_from_swcfg_to_active_legacy
     // Put TX sublink on both ends in High Speed
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         conns[i]->end0->link_handlers->set_tx_mode(conns[i]->end0,
                                                    NVLINK_SUBLINK_STATE_TX_HS,
                                                    flags);
@@ -1085,6 +1180,9 @@ nvlink_core_train_intranode_conns_from_swcfg_to_active_legacy
     // Wait for sublinks to go in High Speed.
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         pollStatus = nvlink_core_poll_sublink_state(conns[i]->end0,
                                                     NVLINK_SUBLINK_STATE_TX_HS,
                                                     NVLINK_SUBLINK_SUBSTATE_TX_STABLE,
@@ -1121,6 +1219,9 @@ nvlink_core_train_intranode_conns_from_swcfg_to_active_legacy
     // Some settings required before moving to ACTIVE
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         _nvlink_core_set_link_pre_active_settings(conns[i]->end0, flags);
         _nvlink_core_set_link_pre_active_settings(conns[i]->end1, flags);
 
@@ -1136,6 +1237,9 @@ nvlink_core_train_intranode_conns_from_swcfg_to_active_legacy
     // Verify link mode HS on the endpoints
     for (i = 0; i < connCount; i++)
     {
+        if (conns[i] == NULL)
+            continue;
+
         pollStatus = nvlink_core_poll_link_state(conns[i]->end1,
                                                  NVLINK_LINKSTATE_HS,
                                                  NVLINK_TRANSITION_HS_TIMEOUT);
@@ -1198,6 +1302,9 @@ _nvlink_core_set_sublink_pre_hs_settings
     NvU32        flags
 )
 {
+    if (link == NULL)
+        return;
+
     //
     // Before training the sublinks to HS, the PROD values must be loaded.
     // On Volta/NVSwitch, the PROD values get loaded by UCODE during DLPL Init.
@@ -1225,6 +1332,9 @@ _nvlink_core_set_link_pre_active_settings
     NvU32        flags
 )
 {
+    if (link == NULL)
+        return;
+
     // Some settings required before moving to ACTIVE
     link->link_handlers->set_dl_link_mode(link, NVLINK_LINKSTATE_PRE_HS, flags);
 }
@@ -1243,6 +1353,9 @@ _nvlink_core_set_link_post_active_settings
     NvU32        flags
 )
 {
+    if (link == NULL)
+        return;
+
     link->link_handlers->training_complete(link);
 
     link->link_handlers->set_tx_mode(link, NVLINK_SUBLINK_STATE_TX_POST_HS, flags);
