@@ -481,6 +481,7 @@ static inline void HsIncrementNextIndex(
 }
 
 static inline void HsChangeSurfaceFlipRefCount(
+    NVDevEvoPtr pDevEvo,
     NVSurfaceEvoPtr pSurfaceEvo,
     NvBool increase)
 {
@@ -488,7 +489,7 @@ static inline void HsChangeSurfaceFlipRefCount(
         if (increase) {
             nvEvoIncrementSurfaceRefCnts(pSurfaceEvo);
         } else {
-            nvEvoDecrementSurfaceRefCnts(pSurfaceEvo);
+            nvEvoDecrementSurfaceRefCnts(pDevEvo, pSurfaceEvo);
         }
     }
 }
