@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2008-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2008-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -130,6 +130,7 @@ typedef NvU8 FLCN_STATUS;
 // because that will affect other binaries and their signatures
 //
 #define FLCN_ERR_CTXSW_ERROR                                               (0x4EU)
+#define FLCN_ERR_SE_SNPKA_HW_CRYPTO_OPERATION_FAILED                       (0x4FU)
 
 // VPR SEC2 task errors
 #define FLCN_ERR_VPR_APP_INVALID_REQUEST_END_ADDR                          (0x51U)
@@ -232,6 +233,8 @@ typedef NvU8 FLCN_STATUS;
 #define FLCN_ERR_HS_APM_FECS_NOT_HALTED                                    (0xCEU)
 #define FLCN_ERR_HS_APM_SCRATCH_PLM_INVALID                                (0xCFU)
 #define FLCN_ERR_HS_APM_SCRATCH_INIT_INVALID                               (0xD0U)
+#define FLCN_ERR_HS_INVALID_KEY                                            (0xD1U)
+#define FLCN_ERR_HS_SWKG_INVALID_SIGNATURE                                 (0xD2U)
 
 
 //
@@ -259,6 +262,7 @@ typedef NvU8 FLCN_STATUS;
 // Warnings.
 #define FLCN_WARN_NOTHING_TO_DO                                            (0xD0U)
 #define FLCN_WARN_NOT_QUERIED                                              (0xD1U)
+#define FLCN_WARN_DATA_NOT_AVAILABLE                                       (0xD2U)
 
 // Queue handling Errors
 #define FLCN_ERR_QUEUE_MGMT_INVALID_UNIT_ID                                (0xE0U)
@@ -283,4 +287,7 @@ typedef NvU8 FLCN_STATUS;
 #define FLCN_ERR_AUTH_GSP_RM_HANDOFF_FAILED                                (0xF8U)
 #define FLCN_ERR_INVALID_WPRMETA_MAGIC_OR_REVISION                         (0xF9U)
 
+// Arithmetic errors
+#define FLCN_ERR_ARITHMETIC_OVERFLOW                                       (0xFAU)
+
 #endif // FLCNRETVAL_H
