@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 200-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 200-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -145,6 +145,7 @@
 #define PCI_VENDOR_ID_CADENCE       0x17CD
 #define PCI_VENDOR_ID_ARM           0x13B5
 #define PCI_VENDOR_ID_HYGON         0x1D94
+#define PCI_VENDOR_ID_ALIBABA       0x1DED
 
 #define NV_PCI_DEVID_DEVICE                    31:16  /* RW--F */
 #define NV_PCI_SUBID_DEVICE                    31:16  /* RW--F */
@@ -156,6 +157,7 @@
 ///////////////////////////////////////////////////////////////////////////////////////////
 
 #define NV_PCI_DEVID_DEVICE_PG171_SKU200_PG179_SKU220 0x25B6 /* NVIDIA A16 / NVIDIA A2 */
+#define NV_PCI_DEVID_DEVICE_PG189_SKU600              0x1EBA
 
 ///////////////////////////////////////////////////////////////////////////////////////////
 //
@@ -466,6 +468,9 @@
 #define ESC_4000_G4_DEVID                  0xA1C1
 #define ESC_4000_G4_SSDEVID                0x871E
 
+// Lenovo Tomcat Workstation
+#define LENOVO_TOMCAT_DEVID                0x1B81
+#define LENOVO_TOMCAT_SSDEVID              0x104e
 
 // NVIDIA C51
 #define NVIDIA_C51_DEVICE_ID_MIN                        0x2F0
@@ -633,7 +638,6 @@ enum {
 ,     CS_MELLANOX_BLUEFIELD
 ,     CS_AMAZON_GRAVITRON2
 ,     CS_FUJITSU_A64FX
-,     CS_PHYTIUM_FT2000
 ,     CS_AMPERE_ALTRA
 ,     CS_ARM_NEOVERSEN1
 ,     CS_MARVELL_OCTEON_CN96XX
@@ -642,6 +646,7 @@ enum {
 ,     CS_HYGON_C86
 ,     CS_PHYTIUM_S2500
 ,     CS_MELLANOX_BLUEFIELD2
+,     CS_ALIBABA_YITIAN
 ,     CS_INTEL_1B81
 ,     CS_INTEL_18DC
 ,     CS_INTEL_7A04
