@@ -801,6 +801,8 @@ typedef struct UvmGpuFaultInfo_tag
     NvHandle faultBufferHandle;
 } UvmGpuFaultInfo;
 
+struct Device;
+
 typedef struct UvmGpuPagingChannel_tag
 {
     struct gpuDevice  *device;
@@ -808,6 +810,7 @@ typedef struct UvmGpuPagingChannel_tag
     NvHandle          channelHandle;
     NvHandle          errorNotifierHandle;
     void              *pushStreamSp;
+    struct Device     *pDevice;
 } UvmGpuPagingChannel, *UvmGpuPagingChannelHandle;
 
 typedef struct UvmGpuAccessCntrInfo_tag
@@ -868,6 +871,16 @@ typedef struct UvmGpuAccessCntrConfig_tag
     NvU32 threshold;
 } UvmGpuAccessCntrConfig;
 
+//
+// When modifying this enum, make sure they are compatible with the mirrored
+// MEMORY_PROTECTION enum in phys_mem_allocator.h.
+//
+typedef enum UvmPmaGpuMemoryType_tag
+{
+    UVM_PMA_GPU_MEMORY_TYPE_UNPROTECTED = 0,
+    UVM_PMA_GPU_MEMORY_TYPE_PROTECTED   = 1
+} UVM_PMA_GPU_MEMORY_TYPE;
+
 typedef UvmGpuChannelInfo gpuChannelInfo;
 typedef UvmGpuChannelAllocParams gpuChannelAllocParams;
 typedef UvmGpuCaps gpuCaps;
