@@ -188,7 +188,7 @@ rmGpuLockAlloc(NvU32 gpuInst)
                       NV_ERR_INVALID_STATE);
 
     // TODO: RM-1492 MODS does not hold API lock when allocating GPUs.
-    NV_ASSERT(rmApiLockIsOwner());
+    NV_ASSERT(rmapiLockIsOwner());
 
     // allocate intr mask lock
     status = rmIntrMaskLockAlloc(gpuInst);
@@ -294,7 +294,7 @@ rmGpuLockFree(NvU32 gpuInst)
     // validate gpuInst argument
     NV_ASSERT_OR_RETURN_VOID((gpuInst < NV_MAX_DEVICES));
     // TODO: RM-1492 MODS does not hold API lock when allocating GPUs.
-    NV_ASSERT(rmApiLockIsOwner());
+    NV_ASSERT(rmapiLockIsOwner());
 
     pGpuLock = &rmGpuLockInfo.gpuLocks[gpuInst];
 
@@ -409,12 +409,20 @@ static void _gpuLocksAcquireDisableInterrupts(NvU32 gpuInst, NvU32 flags)
     if (osLockShouldToggleInterrupts(pGpu))
     {
         Intr *pIntr = GPU_GET_INTR(pGpu);
-        NvBool isIsr = !!(flags & GPUS_LOCK_FLAGS_COND_ACQUIRE);
+        NvBool isIsr = !!(flags & GPU_LOCK_FLAGS_COND_ACQUIRE);
         NvBool bBcEnabled = gpumgrGetBcEnabledStatus(pGpu);
 
         // Always disable intrs for cond code
         gpumgrSetBcEnabledStatus(pGpu, NV_FALSE);
 
+        // Note: SWRL is enabled only for vGPU, and is disabled otherwise.
+        if (pGpu->getProperty(pGpu, PDB_PROP_GPU_SWRL_GRANULAR_LOCKING))
+        {
+            // Disable the RM callback timer interrupt
+            OBJTMR *pTmr = GPU_GET_TIMER(pGpu);
+            tmrRmCallbackIntrDisable(pTmr, pGpu);
+        }
+
         osDisableInterrupts(pGpu, isIsr);
 
         if ((pIntr != NULL) && pIntr->getProperty(pIntr, PDB_PROP_INTR_USE_INTR_MASK_FOR_LOCKING) &&
@@ -477,7 +485,7 @@ _rmGpuLocksAcquire(NvU32 gpuMask, NvU32 flags, NvU32 module, void *ra, NvU32 *pG
     NvBool    bLockAll = NV_FALSE;
 
     bHighIrql = (portSyncExSafeToSleep() == NV_FALSE);
-    bCondAcquireCheck = ((flags & GPUS_LOCK_FLAGS_COND_ACQUIRE) != 0);
+    bCondAcquireCheck = ((flags & GPU_LOCK_FLAGS_COND_ACQUIRE) != 0);
 
     if (pGpuLockedMask)
         *pGpuLockedMask = 0;
@@ -555,12 +563,11 @@ _rmGpuLocksAcquire(NvU32 gpuMask, NvU32 flags, NvU32 module, void *ra, NvU32 *pG
         // If we own a higher order lock than one of the needed ones, we are
         // violating the locking order and need to do a conditional acquire
         // clz32(0) == ctz(0) == 32:
-        //    owned=0b00110000, needed=0b00001100: (4  < (32-28)), bCond=FALSE
-        //    owned=0b00110010, needed=0b00001100: (1  < (32-28)), bCond=TRUE
-        //    owned=0b00010000, needed=0b11000011: (4  < (32-24)), bCond=TRUE
-        //    owned=0b00000000, needed=0b00001100: (32 < (32-28)), bCond=FALSE
-        //    owned=0b00000001, needed=0b00000000: (0  < (32-32)), bCond=FALSE
-        if (portUtilCountTrailingZeros32(ownedMask) < (32-portUtilCountLeadingZeros32(gpuMask)))
+        //    owned=0b00001100, needed=0b00110000: ((32-28) >  4), bCond=FALSE
+        //    owned=0b00001100, needed=0b00110010: ((32-28) >  1), bCond=TRUE
+        //    owned=0b11000011, needed=0b00010000: ((32-24) >  4), bCond=TRUE
+        //    owned=0b00000000, needed=0b00000001: ((32-32) >  0), bCond=FALSE
+        if ((32-portUtilCountLeadingZeros32(ownedMask)) > portUtilCountTrailingZeros32(gpuMask))
         {
             bCondAcquireCheck = NV_TRUE;
         }
@@ -894,8 +901,11 @@ rmGpuGroupLockAcquire
     {
         GPU_MASK deviceGpuMask = 0;
         rmGpuGroupLockGetMask(gpuInst, GPU_LOCK_GRP_DEVICE, &deviceGpuMask);
+        //
         // Verify that we actually locked *this* device, not all others.
-        if ((*pGpuMask & deviceGpuMask) != deviceGpuMask)
+        // Check *held* locks not *acquired* locks in case of SAFE_LOCK_UPGRADE.
+        //
+        if ((rmGpuLocksGetOwnedMask() & deviceGpuMask) != deviceGpuMask)
         {
             //
             // On Windows, at high IRQL we can't signal the semaphore. So we
@@ -1119,6 +1129,22 @@ static void _gpuLocksReleaseEnableInterrupts(NvU32 gpuInst, NvU32 flags)
         }
         osEnableInterrupts(pGpu);
 
+        // Note: SWRL is enabled only for vGPU, and is disabled otherwise.
+        if (pGpu->getProperty(pGpu, PDB_PROP_GPU_SWRL_GRANULAR_LOCKING))
+        {
+            // Enable the alarm interrupt. Rearm MSI when timer interrupt is pending.
+            OBJTMR *pTmr = GPU_GET_TIMER(pGpu);
+            NvU32   retVal;
+
+            tmrRmCallbackIntrEnable(pTmr, pGpu);
+            tmrGetIntrStatus_HAL(pGpu, pTmr, &retVal, NULL);
+            if (retVal != 0)
+            {
+                KernelBif *pKernelBif = GPU_GET_KERNEL_BIF(pGpu);
+                kbifCheckAndRearmMSI(pGpu, pKernelBif);
+            }
+        }
+
         gpumgrSetBcEnabledStatus(pGpu, bBcEnabled);
     }
 }
