@@ -212,17 +212,11 @@ kbusDestroyPeerAccess_GM200
     NvU32      peerNum
 )
 {
-    if (pKernelBus->p2pPcie.busPeer[peerNum].pRemoteWMBoxMemDesc != NULL)
-    {
-        memdescDestroy(pKernelBus->p2pPcie.busPeer[peerNum].pRemoteWMBoxMemDesc);
-        pKernelBus->p2pPcie.busPeer[peerNum].pRemoteWMBoxMemDesc = NULL;
-    }
+    memdescDestroy(pKernelBus->p2pPcie.busPeer[peerNum].pRemoteWMBoxMemDesc);
+    pKernelBus->p2pPcie.busPeer[peerNum].pRemoteWMBoxMemDesc = NULL;
 
-    if (pKernelBus->p2pPcie.busPeer[peerNum].pRemoteP2PDomMemDesc != NULL)
-    {
-        memdescDestroy(pKernelBus->p2pPcie.busPeer[peerNum].pRemoteP2PDomMemDesc);
-        pKernelBus->p2pPcie.busPeer[peerNum].pRemoteP2PDomMemDesc = NULL;
-    }
+    memdescDestroy(pKernelBus->p2pPcie.busPeer[peerNum].pRemoteP2PDomMemDesc);
+    pKernelBus->p2pPcie.busPeer[peerNum].pRemoteP2PDomMemDesc = NULL;
 }
 
 /*!
@@ -362,6 +356,11 @@ kbusCreateP2PMappingForMailbox_GM200
     NV2080_CTRL_INTERNAL_HSHUB_PEER_CONN_CONFIG_PARAMS params;
     NvU32 gpuInst0, gpuInst1;
 
+    if (IS_VIRTUAL(pGpu0) || IS_VIRTUAL(pGpu1))
+    {
+        return NV_ERR_NOT_SUPPORTED;
+    }
+
     if (peer0 == NULL || peer1 == NULL)
     {
         return NV_ERR_INVALID_ARGUMENT;
@@ -669,6 +668,11 @@ kbusRemoveP2PMappingForMailbox_GM200
     NvU32 gpuInst0 = gpuGetInstance(pGpu0);
     NvU32 gpuInst1 = gpuGetInstance(pGpu1);
 
+    if (IS_VIRTUAL(pGpu0) || IS_VIRTUAL(pGpu1))
+    {
+        return NV_ERR_NOT_SUPPORTED;
+    }
+
     // a non-existent mapping
     if(peer0 == BUS_INVALID_PEER ||
        peer1 == BUS_INVALID_PEER)
