@@ -22,6 +22,7 @@
  */
 
 #include "core/core.h"
+#include "core/locks.h"
 #include <rmp2pdefines.h>
 #include "gpu/gpu.h"
 #include "gpu/mem_sys/kern_mem_sys.h"
@@ -62,20 +63,17 @@ NV_STATUS RmP2PValidateSubDevice
  * @brief frees given third party p2p memory extent
  */
 static
-NV_STATUS _freeMappingExtentInfo
+void _freeMappingExtentInfo
 (
     PCLI_THIRD_PARTY_P2P_MAPPING_EXTENT_INFO pExtentInfo
 )
 {
     if (pExtentInfo == NULL)
-        return NV_OK;
+        return;
 
-    if (pExtentInfo->pMemDesc != NULL)
-        memdescDestroy(pExtentInfo->pMemDesc);
+    memdescDestroy(pExtentInfo->pMemDesc);
 
     portMemFree(pExtentInfo);
-
-    return NV_OK;
 }
 
 /*!
@@ -159,6 +157,8 @@ NV_STATUS _createThirdPartyP2PMappingExtent
     PCLI_THIRD_PARTY_P2P_MAPPING_EXTENT_INFO pExtentInfoTmp;
     RsClient *pClient;
     Device *pDevice;
+    NvBool bGpuLockTaken = (rmDeviceGpuLockIsOwner(gpuGetInstance(pGpu)) ||
+                            rmGpuLockIsOwner());
 
     status = serverGetClientUnderLock(&g_resServ, hClient, &pClient);
     NV_ASSERT_OR_RETURN(status == NV_OK, NV_ERR_INVALID_ARGUMENT);
@@ -204,9 +204,24 @@ NV_STATUS _createThirdPartyP2PMappingExtent
     }
     else
     {
-        status = kbusMapFbAperture_HAL(pGpu, pKernelBus, (*ppExtentInfo)->pMemDesc, 0,
-                                       &fbApertureOffset, &fbApertureMapLength,
-                                       BUS_MAP_FB_FLAGS_MAP_UNICAST, hClient);
+        if (!bGpuLockTaken)
+        {
+            status = rmDeviceGpuLocksAcquire(pGpu, GPUS_LOCK_FLAGS_NONE,
+                                             RM_LOCK_MODULES_P2P);
+            NV_ASSERT_OR_GOTO(status == NV_OK, out);
+        }
+
+        status = kbusMapFbAperture_HAL(pGpu, pKernelBus,
+                                       (*ppExtentInfo)->pMemDesc, 0,
+                                       &fbApertureOffset,
+                                       &fbApertureMapLength,
+                                       BUS_MAP_FB_FLAGS_MAP_UNICAST,
+                                       hClient);
+
+        if (!bGpuLockTaken)
+        {
+            rmDeviceGpuLocksRelease(pGpu, GPUS_LOCK_FLAGS_NONE, NULL);
+        }
     }
     if (status != NV_OK)
     {
@@ -249,15 +264,33 @@ out:
             }
             else
             {
+                if (!bGpuLockTaken)
+                {
+                    tmpStatus = rmDeviceGpuLocksAcquire(pGpu, GPUS_LOCK_FLAGS_NONE,
+                                                        RM_LOCK_MODULES_P2P);
+                    NV_ASSERT(tmpStatus == NV_OK);
+
+                    if (tmpStatus != NV_OK)
+                    {
+                        goto cleanup;
+                    }
+                }
+
                 tmpStatus = kbusUnmapFbAperture_HAL(pGpu, pKernelBus,
                                                     (*ppExtentInfo)->pMemDesc,
                                                     fbApertureOffset,
                                                     fbApertureMapLength,
                                                     BUS_MAP_FB_FLAGS_MAP_UNICAST);
+
+                if (!bGpuLockTaken)
+                {
+                    rmDeviceGpuLocksRelease(pGpu, GPUS_LOCK_FLAGS_NONE, NULL);
+                }
             }
             NV_ASSERT(tmpStatus == NV_OK);
         }
 
+cleanup:
         _freeMappingExtentInfo(*ppExtentInfo);
     }
     return status;
@@ -338,6 +371,11 @@ NV_STATUS RmThirdPartyP2PMappingFree
     PCLI_THIRD_PARTY_P2P_MAPPING_EXTENT_INFO pExtentInfoNext = NULL;
     RsClient                           *pClient;
     Device                             *pDevice;
+    NvBool                              bGpuLockTaken;
+    NvBool                              bVgpuRpc;
+
+    bGpuLockTaken = (rmDeviceGpuLockIsOwner(gpuGetInstance(pGpu)) ||
+                     rmGpuLockIsOwner());
 
     NV_ASSERT_OR_RETURN((pGpu != NULL), NV_ERR_INVALID_ARGUMENT);
     NV_ASSERT_OR_RETURN((pMappingInfo != NULL), NV_ERR_INVALID_ARGUMENT);
@@ -357,6 +395,15 @@ NV_STATUS RmThirdPartyP2PMappingFree
     length = pMappingInfo->length;
     address = pMappingInfo->address;
 
+    bVgpuRpc = IS_VIRTUAL(pGpu) && gpuIsWarBug200577889SriovHeavyEnabled(pGpu);
+
+    if (!bGpuLockTaken && !bVgpuRpc)
+    {
+        status = rmDeviceGpuLocksAcquire(pGpu, GPUS_LOCK_FLAGS_NONE,
+                                         RM_LOCK_MODULES_P2P);
+        NV_ASSERT_OK_OR_RETURN(status);
+    }
+
     for(pExtentInfo = pMappingInfo->pStart; (pExtentInfo != NULL) && (length != 0);
         pExtentInfo = pExtentInfoNext)
     {
@@ -369,7 +416,7 @@ NV_STATUS RmThirdPartyP2PMappingFree
         pExtentInfo->refCount--;
         if (pExtentInfo->refCount == 0)
         {
-            if (IS_VIRTUAL(pGpu) && gpuIsWarBug200577889SriovHeavyEnabled(pGpu))
+            if (bVgpuRpc)
             {
                 NV_RM_RPC_UNMAP_MEMORY(pGpu, hClient,
                                        RES_GET_HANDLE(pDevice),
@@ -379,11 +426,11 @@ NV_STATUS RmThirdPartyP2PMappingFree
             }
             else
             {
-               status = kbusUnmapFbAperture_HAL(pGpu, pKernelBus,
-                                                pExtentInfo->pMemDesc,
-                                                pExtentInfo->fbApertureOffset,
-                                                pExtentInfo->length,
-                                                BUS_MAP_FB_FLAGS_MAP_UNICAST);
+                status = kbusUnmapFbAperture_HAL(pGpu, pKernelBus,
+                                                 pExtentInfo->pMemDesc,
+                                                 pExtentInfo->fbApertureOffset,
+                                                 pExtentInfo->length,
+                                                 BUS_MAP_FB_FLAGS_MAP_UNICAST);
             }
             NV_ASSERT(status == NV_OK);
 
@@ -393,6 +440,12 @@ NV_STATUS RmThirdPartyP2PMappingFree
             _freeMappingExtentInfo(pExtentInfo);
         }
     }
+
+    if (!bGpuLockTaken && !bVgpuRpc)
+    {
+        rmDeviceGpuLocksRelease(pGpu, GPUS_LOCK_FLAGS_NONE, NULL);
+    }
+
     NV_ASSERT(length == 0);
 
     pMappingInfo->pStart = NULL;
