@@ -103,8 +103,10 @@ kfspInitRegistryOverrides
         }
     }
 
-    // Inst-in-sys must only set up FRTS in SYSMEM. This includes FB broken.
-    if (pGpu->getProperty(pGpu, PDB_PROP_GPU_IS_ALL_INST_IN_SYSMEM))
+    // Inst-in-sys must only set up FRTS in SYSMEM. This includes FB broken and cache only.
+    if (pGpu->getProperty(pGpu, PDB_PROP_GPU_IS_ALL_INST_IN_SYSMEM) ||
+        pGpu->getProperty(pGpu, PDB_PROP_GPU_BROKEN_FB) ||
+        gpuIsCacheOnlyModeEnabled(pGpu))
     {
         pKernelFsp->setProperty(pKernelFsp, PDB_PROP_KFSP_DISABLE_FRTS_VIDMEM, NV_TRUE);
     }
@@ -148,11 +150,8 @@ kfspStateDestroy_IMPL
     KernelFsp *pKernelFsp
 )
 {
-    if (pKernelFsp->pCotPayload != NULL)
-    {
-        portMemFree(pKernelFsp->pCotPayload);
-        pKernelFsp->pCotPayload = NULL;
-    }
+    portMemFree(pKernelFsp->pCotPayload);
+    pKernelFsp->pCotPayload = NULL;
 
     if (pKernelFsp->pSysmemFrtsMemdesc != NULL)
     {
@@ -249,7 +248,7 @@ kfspPollForQueueEmpty_IMPL
 {
     RMTIMEOUT timeout;
 
-    gpuSetTimeout(pGpu, GPU_TIMEOUT_DEFAULT, &timeout, 0);
+    gpuSetTimeout(pGpu, GPU_TIMEOUT_DEFAULT, &timeout, GPU_TIMEOUT_FLAGS_OSTIMER | GPU_TIMEOUT_FLAGS_BYPASS_THREAD_STATE);
 
     while (!kfspIsQueueEmpty(pGpu, pKernelFsp))
     {
@@ -315,7 +314,7 @@ kfspPollForResponse_IMPL
     NV_STATUS status = NV_OK;
 
     // Poll for message queue to wait for FSP's reply
-    gpuSetTimeout(pGpu, GPU_TIMEOUT_DEFAULT, &timeout, 0);
+    gpuSetTimeout(pGpu, GPU_TIMEOUT_DEFAULT, &timeout, GPU_TIMEOUT_FLAGS_OSTIMER | GPU_TIMEOUT_FLAGS_BYPASS_THREAD_STATE);
     while (kfspIsMsgQueueEmpty(pGpu, pKernelFsp))
     {
         if (gpuCheckTimeout(pGpu, &timeout) == NV_ERR_TIMEOUT)
@@ -370,6 +369,7 @@ kfspReadMessage
     }
 
     pPacketBuffer = portMemAllocNonPaged(kfspGetRmChannelSize_HAL(pGpu, pKernelFsp));
+    NV_CHECK_OR_RETURN(LEVEL_ERROR, pPacketBuffer != NULL, NV_ERR_NO_MEMORY);
 
     while ((packetState != MCTP_PACKET_STATE_END) && (packetState != MCTP_PACKET_STATE_SINGLE_PACKET))
     {
@@ -486,6 +486,7 @@ kfspSendPacket_IMPL
     // Pad to align size to 4-bytes boundary since EMEMC increments by DWORDS
     paddedSize = NV_ALIGN_UP(packetSize, sizeof(NvU32));
     pBuffer = portMemAllocNonPaged(paddedSize);
+    NV_CHECK_OR_RETURN(LEVEL_ERROR, pBuffer != NULL, NV_ERR_NO_MEMORY);
     portMemSet(pBuffer, 0, paddedSize);
     portMemCopy(pBuffer, paddedSize, pPacket, paddedSize);
 
@@ -540,6 +541,7 @@ kfspSendAndReadMessage_IMPL
     // Allocate buffer of same size as channel
     fspEmemRmChannelSize = kfspGetRmChannelSize_HAL(pGpu, pKernelFsp);
     pBuffer = portMemAllocNonPaged(fspEmemRmChannelSize);
+    NV_CHECK_OR_RETURN(LEVEL_ERROR, pBuffer != NULL, NV_ERR_NO_MEMORY);
     portMemSet(pBuffer, 0, fspEmemRmChannelSize);
 
     //
