@@ -1623,8 +1623,8 @@ dmaUpdateVASpace_GF100
     KernelGmmu *pKernelGmmu = GPU_GET_KERNEL_GMMU(pGpu);
     NvBool      bFillPteMem = !!(flags & DMA_UPDATE_VASPACE_FLAGS_FILL_PTE_MEM);
     NvBool      bUnmap = !bFillPteMem &&
-                         (flags & DMA_UPDATE_VASPACE_FLAGS_UPDATE_VALID) &&
-                         (SF_VAL(_MMU, _PTE_VALID, valid) == NV_MMU_PTE_VALID_FALSE);
+                          (flags & DMA_UPDATE_VASPACE_FLAGS_UPDATE_VALID) &&
+                          (SF_VAL(_MMU, _PTE_VALID, valid) == NV_MMU_PTE_VALID_FALSE);
     NvBool      bIsIndirectPeer;
     VAS_PTE_UPDATE_TYPE update_type;
 
@@ -1635,6 +1635,18 @@ dmaUpdateVASpace_GF100
     readDisable = !!(flags & DMA_UPDATE_VASPACE_FLAGS_SHADER_WRITE_ONLY);
     bIsIndirectPeer = !!(flags & DMA_UPDATE_VASPACE_FLAGS_INDIRECT_PEER);
 
+    if (pGpu->bEnableBar1SparseForFillPteMemUnmap)
+    {
+        OBJGVASPACE *pGVAS = dynamicCast(pVAS, OBJGVASPACE);
+        if (bFillPteMem &&
+            (pGVAS->flags & VASPACE_FLAGS_BAR_BAR1) &&
+            (flags & DMA_UPDATE_VASPACE_FLAGS_UPDATE_VALID) &&
+            (SF_VAL(_MMU, _PTE_VALID, valid) == NV_MMU_PTE_VALID_FALSE))
+        {
+            bSparse = NV_TRUE;
+        }
+    }
+
     //
     // Determine whether we are invalidating or revoking privileges, so we know
     // whether to flush page accesses or not. ReadDisable and writeDisable have
