@@ -31,7 +31,6 @@
 #include "core/core.h"
 #include "gpu/gpu.h"
 #include "gpu/mem_mgr/virt_mem_allocator.h"
-#include "lib/base_utils.h"
 #include "rmapi/control.h"
 #include "gpu/mem_mgr/virt_mem_allocator_common.h"
 #include "os/os.h"
@@ -48,6 +47,7 @@
 #include "ctrl/ctrl208f/ctrl208fdma.h"
 #include "vgpu/rpc.h"
 #include "core/locks.h"
+#include "virtualization/kernel_hostvgpudeviceapi.h"
 #include "gpu/subdevice/subdevice_diag.h"
 #include "gpu/device/device.h"
 #include "gpu/subdevice/subdevice.h"
@@ -96,10 +96,12 @@ dmaAllocMap_IMPL
     p2p = DRF_VAL(OS46, _FLAGS, _P2P_ENABLE, pDmaMappingInfo->Flags);
 
     //
-    // By default ADDR_FABRIC_V2 should be mapped as peer memory. So, don't honor
+    // By default any fabric memory should be mapped as peer memory. So, don't honor
     // any P2P flags.
     //
-    if (memdescGetAddressSpace(pDmaMappingInfo->pMemDesc) == ADDR_FABRIC_V2)
+    if (
+        (memdescGetAddressSpace(pDmaMappingInfo->pMemDesc) == ADDR_FABRIC_MC) ||
+        (memdescGetAddressSpace(pDmaMappingInfo->pMemDesc) == ADDR_FABRIC_V2))
     {
         p2p  = 0;
     }
@@ -262,7 +264,7 @@ deviceCtrlCmdDmaGetPteInfo_IMPL
     CALL_CONTEXT   *pCallContext = resservGetTlsCallContext();
     RmCtrlParams   *pRmCtrlParams = pCallContext->pControlParams->pLegacyParams;
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
 
     NV_CHECK_OK_OR_RETURN(LEVEL_WARNING,
                           vaspaceGetByHandleOrDeviceDefault(RES_GET_CLIENT(pDevice), pRmCtrlParams->hObject,
@@ -296,12 +298,11 @@ deviceCtrlCmdDmaUpdatePde2_IMPL
     NvBool       bBcState   = NV_TRUE;
     CALL_CONTEXT *pCallContext = resservGetTlsCallContext();
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
 
-    if (
-        (pCallContext->secInfo.privLevel < RS_PRIV_LEVEL_KERNEL))
+    if (pCallContext->secInfo.privLevel < RS_PRIV_LEVEL_KERNEL)
     {
-        return NV_ERR_INSUFFICIENT_PERMISSIONS;
+            return NV_ERR_INSUFFICIENT_PERMISSIONS;
     }
 
     if (IS_VIRTUAL_WITHOUT_SRIOV(pGpu))
@@ -365,7 +366,7 @@ deviceCtrlCmdDmaSetVASpaceSize_IMPL
     OBJGPU          *pGpu    = GPU_RES_GET_GPU(pDevice);
     OBJVASPACE      *pVAS    = NULL;
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
 
     NV_CHECK_OK_OR_RETURN(LEVEL_WARNING,
                           vaspaceGetByHandleOrDeviceDefault(RES_GET_CLIENT(pDevice), RES_GET_HANDLE(pDevice),
@@ -424,7 +425,7 @@ deviceCtrlCmdDmaSetPageDirectory_IMPL
     NV_STATUS       status      = NV_OK;
     NvBool          bBcState    = NV_FALSE;
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
 
     if (IS_VIRTUAL(pGpu) || IS_GSP_CLIENT(pGpu))
     {
@@ -522,7 +523,7 @@ deviceCtrlCmdDmaUnsetPageDirectory_IMPL
     NvBool          bBcState      = NV_FALSE;
     NV_STATUS       status        = NV_OK;
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
 
     if (IS_VIRTUAL_WITHOUT_SRIOV(pGpu))
     {
@@ -607,7 +608,7 @@ deviceCtrlCmdDmaSetPteInfo_IMPL
     OBJVASPACE *pVAS    = NULL;
     NV_STATUS   status  = NV_OK;
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
 
     NV_CHECK_OK_OR_RETURN(LEVEL_WARNING,
                           vaspaceGetByHandleOrDeviceDefault(RES_GET_CLIENT(pDevice), RES_GET_HANDLE(pDevice),
@@ -642,7 +643,7 @@ deviceCtrlCmdDmaFlush_IMPL
     FB_CACHE_OP         cacheOp     = FB_CACHE_OP_UNDEFINED;
     NV_STATUS           status      = NV_OK;
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
 
     NV_PRINTF(LEVEL_INFO, "Flush op invoked with target Unit 0x%x\n",
               flushParams->targetUnit);
@@ -714,7 +715,7 @@ deviceCtrlCmdDmaAdvSchedGetVaCaps_IMPL
     const MEMORY_SYSTEM_STATIC_CONFIG *pMemorySystemConfig =
         kmemsysGetStaticConfig(pGpu, GPU_GET_KERNEL_MEMORY_SYSTEM(pGpu));
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner());
 
     NV_CHECK_OK_OR_RETURN(LEVEL_WARNING,
         vaspaceGetByHandleOrDeviceDefault(RES_GET_CLIENT(pDevice), RES_GET_HANDLE(pDevice),
@@ -748,7 +749,7 @@ deviceCtrlCmdDmaGetPdeInfo_IMPL
     CALL_CONTEXT   *pCallContext    = resservGetTlsCallContext();
     RmCtrlParams   *pRmCtrlParams   = pCallContext->pControlParams->pLegacyParams;
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
 
     //
     // vGPU:
@@ -793,7 +794,7 @@ deviceCtrlCmdDmaSetDefaultVASpace_IMPL
     OBJGPU   *pGpu   = GPU_RES_GET_GPU(pDevice);
     NV_STATUS status = NV_OK;
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
 
     NV_ASSERT_OK_OR_RETURN(
         deviceSetDefaultVASpace(
@@ -883,7 +884,7 @@ subdeviceCtrlCmdDmaGetInfo_IMPL
     NvU32       i;
     NvU32       data;
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
 
     // error checck
     if (pDmaInfoParams->dmaInfoTblSize > NV2080_CTRL_DMA_GET_INFO_MAX_ENTRIES)
@@ -933,7 +934,7 @@ deviceCtrlCmdDmaInvalidateTLB_IMPL
     OBJGPU      *pGpu    = GPU_RES_GET_GPU(pDevice);
     OBJVASPACE  *pVAS    = NULL;
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
 
     //
     // vGPU:
@@ -995,7 +996,7 @@ deviceCtrlCmdDmaGetCaps_IMPL
     OBJGPU     *pGpu = GPU_RES_GET_GPU(pDevice);
     VirtMemAllocator *pDma = GPU_GET_DMA(pGpu);
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner());
 
     // sanity check array size
     if (pDmaCapsParams->capsTblSize != NV0080_CTRL_DMA_CAPS_TBL_SIZE)
@@ -1041,7 +1042,7 @@ deviceCtrlCmdDmaEnablePrivilegedRange_IMPL
     NV0080_CTRL_DMA_ENABLE_PRIVILEGED_RANGE_PARAMS *pParams
 )
 {
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
 
     if (pParams->hVASpace != NV01_NULL_OBJECT)
     {
@@ -1186,47 +1187,3 @@ dmaPageArrayGetPhysAddr
 
     return addr;
 }
-
-/*!
- * Determine if the upper half for the comptag can be used for this page.
- *
- * @param[in] pDma      VirtMemAllocator object pointer
- * @param[in] pteIndex  PTE index
- * @param[in] pageSize  Page size
- *
- * @returns Whether the upper half can be used or not.
- */
-NvBool
-dmaUseCompTagLineUpperHalf_IMPL(VirtMemAllocator *pDma, NvU32 pteIndex, NvU32 pageSize)
-{
-    NvBool bUseUpperHalf = NV_FALSE;
-    OBJGPU *pGpu = ENG_GET_GPU(pDma);
-    const MEMORY_SYSTEM_STATIC_CONFIG *pMemorySystemConfig =
-        kmemsysGetStaticConfig(pGpu, GPU_GET_KERNEL_MEMORY_SYSTEM(pGpu));
-
-    if (pDma->getProperty(pDma, PDB_PROP_DMA_ENABLE_FULL_COMP_TAG_LINE))
-    {
-        NvU32 subIndexShift;
-
-        if (pMemorySystemConfig->comprPageShift >= NvU64_LO32(nvLogBase2(pageSize)))
-        {
-            subIndexShift = pMemorySystemConfig->comprPageShift - NvU64_LO32(nvLogBase2(pageSize));
-        }
-        else
-        {
-            NV_ASSERT_OR_RETURN(0, NV_FALSE);
-        }
-
-        //
-        // (SubIndexShift - 1) is half the compression page size (64 KB).
-        // Shifting by that gives the number of PTEs that span a 64 KB page.
-        // The 0th bit gives comptag subindex.
-        //
-        if (subIndexShift)
-        {
-            bUseUpperHalf = !!((pteIndex >> (subIndexShift - 1)) & 1);
-        }
-    }
-
-    return bUseUpperHalf;
-}
