@@ -30,6 +30,7 @@
 #include "gpu/mmu/kern_gmmu.h"
 #include "gpu/bus/kern_bus.h"
 #include "core/locks.h"
+#include "virtualization/kernel_vgpu_mgr.h"
 #include "vgpu/rpc.h"
 #include "core/thread_state.h"
 #include "nvRmReg.h"
@@ -91,18 +92,12 @@ memmgrDestruct_IMPL
 
     for (i = 0; i < NUM_FBSR_TYPES; i++)
     {
-        if (pMemoryManager->pFbsr[i])
-        {
-            objDelete(pMemoryManager->pFbsr[i]);
-            pMemoryManager->pFbsr[i] = NULL;
-        }
+        objDelete(pMemoryManager->pFbsr[i]);
+        pMemoryManager->pFbsr[i] = NULL;
     }
 
-    if (pMemoryManager->pHeap != NULL)
-    {
-        objDelete(pMemoryManager->pHeap);
-        pMemoryManager->pHeap = NULL;
-    }
+    objDelete(pMemoryManager->pHeap);
+    pMemoryManager->pHeap = NULL;
 
     pMemoryManager->MIGMemoryPartitioningInfo.partitionableMemoryRange = NV_RANGE_EMPTY;
 }
@@ -132,7 +127,7 @@ _memmgrInitRegistryOverrides(OBJGPU *pGpu, MemoryManager *pMemoryManager)
     {
         pMemoryManager->bScrubOnFreeEnabled = NV_FALSE;
     }
-    
+
     if ((osReadRegistryDword(pGpu, NV_REG_STR_RM_DISABLE_FAST_SCRUBBER,
                              &data32) == NV_OK) && data32)
     {
@@ -489,6 +484,32 @@ memmgrStateDestroy_IMPL
     // RMCONFIG: only if FBSR engine is enabled
     if (RMCFG_MODULE_FBSR)
     {
+        // Cleanup fbsrReservedRanges
+        if (pMemoryManager->fbsrReservedRanges[FBSR_RESERVED_INST_MEMORY_BEFORE_BAR2PTE] != NULL)
+            memdescDestroy(pMemoryManager->fbsrReservedRanges[FBSR_RESERVED_INST_MEMORY_BEFORE_BAR2PTE]);
+
+        if (pMemoryManager->fbsrReservedRanges[FBSR_RESERVED_INST_MEMORY_AFTER_BAR2PTE] != NULL)
+            memdescDestroy(pMemoryManager->fbsrReservedRanges[FBSR_RESERVED_INST_MEMORY_AFTER_BAR2PTE]);
+
+        if (pMemoryManager->fbsrReservedRanges[FBSR_RESERVED_INST_MEMORY_GSP_HEAP] != NULL)
+            memdescDestroy(pMemoryManager->fbsrReservedRanges[FBSR_RESERVED_INST_MEMORY_GSP_HEAP]);
+
+        if (pMemoryManager->fbsrReservedRanges[FBSR_RESERVED_INST_MEMORY_GSP_NON_WPR] != NULL)
+            memdescDestroy(pMemoryManager->fbsrReservedRanges[FBSR_RESERVED_INST_MEMORY_GSP_NON_WPR]);
+
+        if (pMemoryManager->fbsrReservedRanges[FBSR_RESERVED_INST_MEMORY_GSP_WPR] != NULL)
+            memdescDestroy(pMemoryManager->fbsrReservedRanges[FBSR_RESERVED_INST_MEMORY_GSP_WPR]);
+
+        if (pMemoryManager->fbsrReservedRanges[FBSR_RESERVED_INST_MEMORY_VGA_WORKSPACE] != NULL)
+            memdescDestroy(pMemoryManager->fbsrReservedRanges[FBSR_RESERVED_INST_MEMORY_VGA_WORKSPACE]);
+
+        pMemoryManager->fbsrReservedRanges[FBSR_RESERVED_INST_MEMORY_BEFORE_BAR2PTE] = NULL;
+        pMemoryManager->fbsrReservedRanges[FBSR_RESERVED_INST_MEMORY_AFTER_BAR2PTE]  = NULL;
+        pMemoryManager->fbsrReservedRanges[FBSR_RESERVED_INST_MEMORY_GSP_HEAP]       = NULL;
+        pMemoryManager->fbsrReservedRanges[FBSR_RESERVED_INST_MEMORY_GSP_NON_WPR]    = NULL;
+        pMemoryManager->fbsrReservedRanges[FBSR_RESERVED_INST_MEMORY_GSP_WPR]        = NULL;
+        pMemoryManager->fbsrReservedRanges[FBSR_RESERVED_INST_MEMORY_VGA_WORKSPACE]  = NULL;
+
         for (i = 0; i < NUM_FBSR_TYPES; i++)
         {
             fbsrDestroy_HAL(pGpu, pMemoryManager->pFbsr[i]);
@@ -580,7 +601,7 @@ memmgrCreateHeap_IMPL
 
         pMemoryManager->pHeap = newHeap;
 
-        if (memmgrIsPmaEnabled(pMemoryManager) && 
+        if (memmgrIsPmaEnabled(pMemoryManager) &&
             memmgrIsPmaSupportedOnPlatform(pMemoryManager))
         {
             portMemSet(&pMemoryManager->pHeap->pmaObject, 0, sizeof(pMemoryManager->pHeap->pmaObject));
@@ -1308,13 +1329,13 @@ memmgrAllocateConsoleRegion_IMPL
 
     NV_STATUS status     = NV_OK;
     NvU32     consoleRegionId = 0x0;
-    NvU64     regionSize; 
-    
+    NvU64     regionSize;
+
     if (pMemoryManager->Ram.ReservedConsoleDispMemSize > 0)
     {
-        pConsoleFbRegion->base = pMemoryManager->Ram.fbRegion[consoleRegionId].base; 
+        pConsoleFbRegion->base = pMemoryManager->Ram.fbRegion[consoleRegionId].base;
         pConsoleFbRegion->limit = pMemoryManager->Ram.fbRegion[consoleRegionId].limit;
-        
+
         regionSize = pConsoleFbRegion->limit - pConsoleFbRegion->base + 1;
 
         // Once the console is reserved, we don't expect to reserve it again
@@ -1940,7 +1961,7 @@ memmgrFillComprInfo_IMPL
     pComprInfo->compPageShift = pMemorySystemConfig->comprPageShift;
     pComprInfo->compTagLineMin = compTagStartOffset;
     pComprInfo->compPageIndexLo = (NvU32)(surfOffset >> pComprInfo->compPageShift);
-    pComprInfo->compPageIndexHi = (NvU32)((surfOffset + pageSize * pageCount - 1) >> pComprInfo->compPageShift);
+    pComprInfo->compPageIndexHi = (NvU32)((surfOffset + (NvU64)pageSize * (NvU64)pageCount - 1) >> pComprInfo->compPageShift);
     pComprInfo->compTagLineMultiplier = 1;
 
     return NV_OK;
@@ -1975,7 +1996,16 @@ memmgrGetKindComprForGpu_KERNEL
     }
     else
     {
-        memmgrFillComprInfoUncompressed(pMemoryManager, kind, pComprInfo);
+        if (ctagId == FB_HWRESID_CTAGID_VAL_FERMI(0xcdcdcdcd))
+        {
+            portMemSet(pComprInfo, 0, sizeof(*pComprInfo));
+
+            pComprInfo->kind = memmgrGetUncompressedKind_HAL(pMappingGpu, pMemoryManager, kind, NV_TRUE);
+        }
+        else
+        {
+            memmgrFillComprInfoUncompressed(pMemoryManager, kind, pComprInfo);
+        }
     }
 
     *pKind = pComprInfo->kind;
@@ -2472,7 +2502,7 @@ memmgrPmaInitialize_IMPL
     NV_STATUS status = NV_OK;
     NvBool bNumaEnabled = osNumaOnliningEnabled(pGpu->pOsGpuInfo);
 
-    NV_ASSERT(memmgrIsPmaEnabled(pMemoryManager) && 
+    NV_ASSERT(memmgrIsPmaEnabled(pMemoryManager) &&
               memmgrIsPmaSupportedOnPlatform(pMemoryManager));
 
     if (memmgrIsPmaForcePersistence(pMemoryManager))
@@ -2517,12 +2547,12 @@ memmgrPmaInitialize_IMPL
         {
             if (numaSkipReclaimVal > NV_REG_STR_RM_NUMA_ALLOC_SKIP_RECLAIM_PERCENTAGE_MAX)
             {
-                numaSkipReclaimVal = NV_REG_STR_RM_NUMA_ALLOC_SKIP_RECLAIM_PERCENTAGE_MAX; 
+                numaSkipReclaimVal = NV_REG_STR_RM_NUMA_ALLOC_SKIP_RECLAIM_PERCENTAGE_MAX;
             }
         }
         pmaNumaSetReclaimSkipThreshold(pPma, numaSkipReclaimVal);
     }
-    
+
     return NV_OK;
 }
 
@@ -3012,7 +3042,7 @@ memmgrReserveMemoryForFsp_IMPL
     //
     // If we sent FSP commands to boot ACR, we need to allocate the surfaces
     // used by FSP and ACR as WPR/FRTS here from the reserved heap
-    //   
+    //
     if (pKernelFsp && (!pKernelFsp->getProperty(pKernelFsp, PDB_PROP_KFSP_DISABLE_FRTS_VIDMEM) &&
         (pKernelFsp->getProperty(pKernelFsp, PDB_PROP_KFSP_BOOT_COMMAND_OK))))
     {
@@ -3026,3 +3056,25 @@ memmgrReserveMemoryForFsp_IMPL
     }
     return NV_OK;
 }
+
+NvU64
+memmgrGetVgpuHostRmReservedFb_KERNEL
+(
+    OBJGPU         *pGpu,
+    MemoryManager  *pMemoryManager,
+    NvU32           vgpuTypeId
+)
+{
+    RM_API *pRmApi = GPU_GET_PHYSICAL_RMAPI(pGpu);
+    NV2080_CTRL_INTERNAL_MEMMGR_GET_VGPU_CONFIG_HOST_RESERVED_FB_PARAMS params = {0};
+
+    params.vgpuTypeId = vgpuTypeId;
+    // Send to GSP to get amount of FB reserved for the host
+    NV_ASSERT_OK_OR_RETURN(pRmApi->Control(pRmApi,
+                                           pGpu->hInternalClient,
+                                           pGpu->hInternalSubdevice,
+                                           NV2080_CTRL_CMD_INTERNAL_MEMMGR_GET_VGPU_CONFIG_HOST_RESERVED_FB,
+                                           &params,
+                                           sizeof(params)));
+    return params.hostReservedFb;
+}
\ No newline at end of file
