@@ -703,7 +703,7 @@ subdeviceCtrlCmdFbGetInfo_IMPL
     NvHandle hClient = RES_GET_CLIENT_HANDLE(pSubdevice);
     NvHandle hObject = RES_GET_HANDLE(pSubdevice);
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmDeviceGpuLockIsOwner(pGpu->gpuInstance));
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmDeviceGpuLockIsOwner(pGpu->gpuInstance));
 
     if ((pFbInfoParams->fbInfoListSize == 0) ||
         (NvP64_VALUE(pFbInfoParams->fbInfoList) == NULL))
@@ -731,7 +731,7 @@ subdeviceCtrlCmdFbGetInfoV2_IMPL
     NvHandle hClient = RES_GET_CLIENT_HANDLE(pSubdevice);
     NvHandle hObject = RES_GET_HANDLE(pSubdevice);
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmDeviceGpuLockIsOwner(pGpu->gpuInstance));
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmDeviceGpuLockIsOwner(pGpu->gpuInstance));
 
     if ((pFbInfoParams->fbInfoListSize > NV2080_CTRL_FB_INFO_MAX_LIST_SIZE) ||
         (pFbInfoParams->fbInfoListSize == 0))
@@ -756,7 +756,7 @@ subdeviceCtrlCmdFbGetCarveoutAddressInfo_IMPL
 )
 {
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
 
     pParams->StartAddr = 0x0;
     pParams->SpaceSize = 0x0;
@@ -799,7 +799,7 @@ subdeviceCtrlCmdFbGetGpuCacheAllocPolicy_IMPL
 {
 
     NV_STATUS status = NV_ERR_NOT_SUPPORTED;
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
 
     pGpuCacheAllocPolicyParams->allocPolicy = 0;
 
@@ -847,7 +847,7 @@ subdeviceCtrlCmdFbGetGpuCacheAllocPolicyV2_IMPL
 {
     NV_STATUS status = NV_ERR_NOT_SUPPORTED;
 
-    NV_ASSERT_OR_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner(), NV_ERR_INVALID_LOCK_STATE);
+    NV_ASSERT_OR_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner(), NV_ERR_INVALID_LOCK_STATE);
 
     NV_PRINTF(LEVEL_ERROR, "Failed to get alloc policy.\n");
     return status;
@@ -879,7 +879,7 @@ subdeviceCtrlCmdFbGetCliManagedOfflinedPages_IMPL
     NvU32               pageSize;
     NvU32               numChunks           = 0;
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
     if (!IsSLIEnabled(pGpu))
     {
         if (memmgrIsPmaInitialized(pMemoryManager))
@@ -946,7 +946,7 @@ subdeviceCtrlCmdFbUpdateNumaStatus_IMPL
     if (!RMCFG_FEATURE_PMA)
         return NV_ERR_NOT_SUPPORTED;
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
 
     if (pParams->bOnline)
     {
@@ -992,7 +992,7 @@ subdeviceCtrlCmdFbGetNumaInfo_IMPL
     const MEMORY_SYSTEM_STATIC_CONFIG *pMemorySystemConfig =
         kmemsysGetStaticConfig(pGpu, pKernelMemorySystem);
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
 
     if (pParams->numaOfflineAddressesCount >
         NV_ARRAY_ELEMENTS(pParams->numaOfflineAddresses))
@@ -1135,7 +1135,7 @@ subdeviceCtrlCmdFbFlushGpuCache_IMPL
     NvBool              bWriteback = NV_FALSE;
     NvBool              bInvalidate = NV_FALSE;
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
 
     // Either WriteBack or Invalidate are required for Cache Ops
     if (FLD_TEST_DRF(2080, _CTRL_FB_FLUSH_GPU_CACHE_FLAGS, _WRITE_BACK,
