@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2021-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -132,6 +132,8 @@ knvlinkRemoveMapping_GA100
 )
 {
     NV_STATUS status = NV_OK;
+    NvU32     peerId;
+    NvBool    bBufferReady;
 
     NV2080_CTRL_NVLINK_REMOVE_NVLINK_MAPPING_PARAMS params;
     portMemSet(&params, 0, sizeof(params));
@@ -161,7 +163,35 @@ knvlinkRemoveMapping_GA100
     // the MUX registers and the connection config registers. So, we have
     // to call nvlinkCurrentConfig instead of nvlinkUpdateHshubConfigRegs
     //
-    return knvlinkUpdateCurrentConfig(pGpu, pKernelNvlink);
+    status = knvlinkSyncLinkMasksAndVbiosInfo(pGpu, pKernelNvlink);
+    if (status != NV_OK)
+    {
+        NV_ASSERT(status == NV_OK);
+        return status;
+    }
+
+    if (pKernelNvlink->getProperty(pKernelNvlink, PDB_PROP_KNVLINK_CONFIG_REQUIRE_INITIALIZED_LINKS_CHECK))
+    {
+        FOR_EACH_INDEX_IN_MASK(32, peerId, peerMask)
+        {
+            if (pKernelNvlink->initializedLinks & pKernelNvlink->peerLinkMasks[peerId])
+            {
+                bBufferReady = NV_TRUE;
+                break;
+            }
+        } FOR_EACH_INDEX_IN_MASK_END;
+
+        if (!bBufferReady)
+        {
+            status = knvlinkUpdateCurrentConfig(pGpu, pKernelNvlink);
+        }
+    }
+    else
+    {
+        status = knvlinkUpdateCurrentConfig(pGpu, pKernelNvlink);
+    }
+
+    return status;
 }
 
 /*!
@@ -184,6 +214,7 @@ knvlinkValidateFabricBaseAddress_GA100
 {
     MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
     NvU64          fbSizeBytes;
+    NvU64          fbUpperLimit;
 
     fbSizeBytes = pMemoryManager->Ram.fbTotalMemSizeMb << 20;
 
@@ -202,11 +233,51 @@ knvlinkValidateFabricBaseAddress_GA100
     // Align fbSize to mapslot size.
     fbSizeBytes = RM_ALIGN_UP(fbSizeBytes, NVBIT64(36));
 
+
+    // Check for integer overflow
+    if (!portSafeAddU64(fabricBaseAddr, fbSizeBytes, &fbUpperLimit))
+    {
+        return NV_ERR_INVALID_ARGUMENT;
+    }
+
     // Make sure the address range doesn't go beyond the limit, (2K * 64GB).
-    if ((fabricBaseAddr + fbSizeBytes) > NVBIT64(47))
+    if (fbUpperLimit > NVBIT64(47))
     {
         return NV_ERR_INVALID_ARGUMENT;
     }
 
     return NV_OK;
 }
+
+/*!
+ * @brief   Checks to see if the GPU is a reduced nvlink config
+ *
+ * @param[in] pGpu            OBJGPU pointer
+ * @param[in] pKernelNvlink   KernelNvlink pointer
+ *
+ * @returns On the gpu being a reduced config, NV_TRUE.
+ *          otherwise , returns NV_FALSE.
+ */
+NvBool
+knvlinkIsGpuReducedNvlinkConfig_GA100
+(
+    OBJGPU       *pGpu,
+    KernelNvlink *pKernelNvlink
+)
+{
+    NV2080_CTRL_NVLINK_IS_REDUCED_CONFIG_PARAMS params;
+    NV_STATUS status;
+
+    portMemSet(&params, 0, sizeof(params));
+
+    status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
+                             NV2080_CTRL_CMD_NVLINK_IS_REDUCED_CONFIG,
+                             (void *)&params, sizeof(params));
+    if (status != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR, "Failed to execute GSP-RM GPC to get if the gpu has a reduced Nvlink config\n");
+        return NV_FALSE;
+    }
+
+    return params.bReducedNvlinkConfig;
+}
