@@ -193,6 +193,12 @@ knvlinkConstructEngine_IMPL
         return status;
     }
 
+    // 
+    // When GSP inform about link error occurs on this GPU
+    // it will updated to NV_TRUE
+    //
+    pKernelNvlink->bIsGpuDegraded = NV_FALSE;
+
     //
     // Create MAX KernelIoctrl objects.
     // Later in knvlinkStatePreInit_IMPL, we will remove the objects for
@@ -334,6 +340,7 @@ knvlinkStateLoad_IMPL
                                                !kmigmgrIsMIGNvlinkP2PSupported(pGpu, pKernelMIGManager));
     NvU32             preInitializedLinks;
     NvU32             i;
+    OBJTMR            *pTmr = GPU_GET_TIMER(pGpu);
 
     //
     // If we are on the resume path, nvlinkIsPresent will not be called,
@@ -545,6 +552,10 @@ knvlinkStateLoad_IMPL
         sysEnableExternalFabricMgmt(pSys);
         sysForceInitFabricManagerState(pSys);
     }
+    if (GPU_IS_NVSWITCH_DETECTED(pGpu))
+    {
+        sysEnableExternalFabricMgmt(pSys);
+    }
 
     //
     // WAR Bug# 3261027: Sync-up External Fabric Management status with GSP-RM.
@@ -562,6 +573,7 @@ knvlinkStateLoad_IMPL
     // If we are running on CPU-RM or monolithic, process SYSMEM links, if present
     // on the system.
     //
+
     status = _knvlinkProcessSysmemLinks(pGpu, pKernelNvlink,
                         (preInitializedLinks != pKernelNvlink->initializedLinks));
     if (status != NV_OK)
@@ -578,7 +590,7 @@ knvlinkStateLoad_IMPL
     //
     if (!(flags & GPU_STATE_FLAGS_PRESERVING))
     {
-        if ((status = kbusInitFla_HAL(pGpu, GPU_GET_KERNEL_BUS(pGpu), 0, 0)) != NV_OK)
+        if ((status = kbusCheckFlaSupportedAndInit_HAL(pGpu, GPU_GET_KERNEL_BUS(pGpu), 0, 0)) != NV_OK)
         {
             NV_PRINTF(LEVEL_ERROR, "Init FLA failed, status:0x%x\n", status);
             NV_ASSERT(status == NV_OK);
@@ -641,6 +653,21 @@ knvlinkStateLoad_IMPL
         FOR_EACH_INDEX_IN_MASK_END;
     }
 
+    FOR_EACH_INDEX_IN_MASK(32, i, pKernelNvlink->enabledLinks)
+    {
+        status = tmrEventCreate(pTmr, &pKernelNvlink->nvlinkLinks[i].pTmrEvent,
+                            ioctrlFaultUpTmrHandler, NULL,
+                            TMR_FLAGS_NONE);
+        if (status != NV_OK)
+        {
+           NV_PRINTF(LEVEL_ERROR,
+                          "Failed to create TmrEvent for Link %d\n", i);
+        }
+    }
+    FOR_EACH_INDEX_IN_MASK_END;
+
+    listInit(&pKernelNvlink->faultUpLinks, portMemAllocatorGetGlobalNonPaged());
+
 knvlinkStateLoad_end:
 
     if (status != NV_OK)
@@ -852,7 +879,8 @@ knvlinkStatePostUnload_IMPL
     // platforms (in discussion with ARCH for non-NVSwitch platforms).
     //
     if (pSys->getProperty(pSys, PDB_PROP_SYS_NVSWITCH_IS_PRESENT) ||
-        knvlinkIsNvswitchProxyPresent(pGpu, pKernelNvlink))
+        knvlinkIsNvswitchProxyPresent(pGpu, pKernelNvlink)        ||
+        (GPU_IS_NVSWITCH_DETECTED(pGpu)))
     {
         knvlinkRemoveMapping_HAL(pGpu, pKernelNvlink, NV_FALSE,
                                  ((1 << NVLINK_MAX_PEERS_SW) - 1),
@@ -903,12 +931,15 @@ knvlinkStatePostUnload_IMPL
         if (!API_GPU_IN_RESET_SANITY_CHECK(pRemoteGpu))
         {
             KernelNvlink *pRemoteKernelNvlink = GPU_GET_KERNEL_NVLINK(pRemoteGpu);
+
             pRemoteKernelNvlink->disconnectedLinkMask |= NVBIT(pKernelNvlink->nvlinkLinks[linkId].remoteEndInfo.linkNumber);
         }
     }
     FOR_EACH_INDEX_IN_MASK_END;
 #endif
 
+    listDestroy(&pKernelNvlink->faultUpLinks);
+
 knvlinkStatePostUnload_end:
 
     _knvlinkPurgeState(pGpu, pKernelNvlink);
@@ -933,13 +964,28 @@ _knvlinkPurgeState
 {
     KernelIoctrl *pKernelIoctrl = NULL;
     NvU32         ioctrlIdx;
-
 #if defined(INCLUDE_NVLINK_LIB)
 
+    NvU32         linkId;
+    OBJTMR       *pTmr = GPU_GET_TIMER(pGpu);
     KernelMIGManager *pKernelMIGManager = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
     NvBool bMIGNvLinkP2PDisabled = ((pKernelMIGManager != NULL) &&
                           !kmigmgrIsMIGNvlinkP2PSupported(pGpu, pKernelMIGManager));
-    
+
+    FOR_EACH_INDEX_IN_MASK(32, linkId, pKernelNvlink->enabledLinks)
+    {
+        if ((pKernelNvlink->nvlinkLinks[linkId].pTmrEvent != NULL) && (pTmr != NULL))
+        {
+            if (tmrEventOnList(pTmr, pKernelNvlink->nvlinkLinks[linkId].pTmrEvent))
+            {
+                 tmrEventCancel(pTmr, pKernelNvlink->nvlinkLinks[linkId].pTmrEvent);
+            }
+            tmrEventDestroy(pTmr, pKernelNvlink->nvlinkLinks[linkId].pTmrEvent);
+            pKernelNvlink->nvlinkLinks[linkId].pTmrEvent = NULL;
+        }
+    }
+    FOR_EACH_INDEX_IN_MASK_END;
+
     // RM disables NVLink at runtime in Hopper so device un-registration can't be skipped
     if (!IsGH100orBetter(pGpu))
     {
@@ -963,8 +1009,6 @@ _knvlinkPurgeState
     {
         if (pKernelNvlink->pNvlinkDev)
         {
-            NvU32 linkId;
-
             // Un-register the links from nvlink core library
             FOR_EACH_INDEX_IN_MASK(32, linkId, pKernelNvlink->enabledLinks)
             {
@@ -1000,15 +1044,97 @@ _knvlinkPurgeState_end:
     }
 
     // Destroy the chiplib configuration memory
-    if (pKernelNvlink->pLinkConnection)
-    {
-        portMemFree(pKernelNvlink->pLinkConnection);
-        pKernelNvlink->pLinkConnection = NULL;
-    }
+    portMemFree(pKernelNvlink->pLinkConnection);
+    pKernelNvlink->pLinkConnection = NULL;
 
     return NV_OK;
 }
 
+/*!
+ * @brief Degraded Mode will be set if other end of the linkId
+ *        is not degraded.
+ *        Once degraded destroy the RM NVLink SW state
+ *
+ * @param[in] pGpu           OBJGPU pointer
+ * @param[in] pKernelNvlink  KernelNvlink pointer
+ * @param[in] linkId         linkId of the error link
+ *
+ */
+void
+knvlinkSetDegradedMode_IMPL
+(
+    OBJGPU       *pGpu,
+    KernelNvlink *pKernelNvlink,
+    NvU32         linkId
+)
+{
+    NvU32         status = NV_ERR_GENERIC;
+    NvU32         gpuInstance;
+    OBJGPU       *pRemoteGpu = NULL;
+    KernelNvlink *pRemoteKernelNvlink = NULL;
+
+    if (!pKernelNvlink)
+    {
+        NV_PRINTF(LEVEL_ERROR,
+                "Failed to get Local Nvlink info for linkId %d to update Degraded GPU%d status\n", 
+                linkId, pGpu->gpuInstance);
+                
+        return; 
+    }
+
+    if(pKernelNvlink->bIsGpuDegraded)
+    {
+        return; 
+    }
+
+    //Find the remote GPU/NVLink attached to this link, if any
+    for (gpuInstance = 0; gpuInstance < NV_MAX_DEVICES; gpuInstance++)
+    {
+        if (pKernelNvlink->peerLinkMasks[gpuInstance] & NVBIT(linkId))
+        {
+            pRemoteGpu = gpumgrGetGpu(gpuInstance);
+            break;
+        }
+    }
+
+    if (pRemoteGpu == NULL)
+    {
+        NV_PRINTF(LEVEL_ERROR,
+                "Failed to get Remote GPU info for linkId %d to update Degraded GPU%d status\n", 
+                linkId, pGpu->gpuInstance);
+                
+        return; 
+    }
+
+    pRemoteKernelNvlink = GPU_GET_KERNEL_NVLINK(pRemoteGpu);
+    if (!pRemoteKernelNvlink)
+    {
+        NV_PRINTF(LEVEL_ERROR,
+                "Failed to get Remote Nvlink info for linkId %d to update Degraded GPU%d status\n", 
+                linkId, pGpu->gpuInstance);
+                
+        return; 
+    }
+
+    if (pRemoteKernelNvlink->bIsGpuDegraded == NV_FALSE)
+    {
+        pKernelNvlink->bIsGpuDegraded = NV_TRUE;
+        NV_PRINTF(LEVEL_ERROR,
+                "GPU%d marked Degraded for error on linkId %d \n", 
+                pGpu->gpuInstance, linkId);
+
+        // shutdown all the links on this GPU
+        status = knvlinkCoreShutdownDeviceLinks(pGpu, pKernelNvlink, NV_TRUE);
+        if (status != NV_OK)
+        {
+           NV_PRINTF(LEVEL_ERROR,
+                     "failed to shutdown links on degraded GPU%d\n", pGpu->gpuInstance);
+        }
+    }
+    
+    return;
+}
+
 void
 knvlinkDestruct_IMPL
 (
