@@ -22,7 +22,7 @@
  */
 
 
-/***************************** HW State Rotuines ***************************\
+/***************************** HW State Routines ***************************\
 *                                                                           *
 *         Fabric Virtual Address Space Function Definitions.                *
 *                                                                           *
@@ -31,10 +31,10 @@
 #include "gpu/mmu/kern_gmmu.h"
 #include "mem_mgr/vaspace.h"
 #include "mem_mgr/fabric_vaspace.h"
+#include "gpu/mem_mgr/mem_mgr.h"
 #include "mem_mgr/gpu_vaspace.h"
 #include "gpu/mem_mgr/virt_mem_allocator_common.h"
 #include "os/os.h"
-#include "gpu/mem_mgr/mem_desc.h"
 #include "gpu/bus/kern_bus.h"
 #include "kernel/gpu/fifo/kernel_fifo.h"
 #include "kernel/gpu/nvlink/kernel_nvlink.h"
@@ -51,6 +51,8 @@
 #include "vgpu/vgpu_events.h"
 #include "mem_mgr/virt_mem_mgr.h"
 
+#include "published/ampere/ga100/dev_mmu.h"
+
 
 
 //
@@ -73,6 +75,11 @@ _fabricvaspaceBindInstBlk
 
     INST_BLK_INIT_PARAMS instblkParams;
 
+    if (!pKernelBus->flaInfo.bToggleBindPoint)
+    {
+        return NV_OK;
+    }
+
     if (gvaspaceIsInUse(dynamicCast(pKernelBus->flaInfo.pFlaVAS, OBJGVASPACE)))
     {
         NV_PRINTF(LEVEL_ERROR,
@@ -156,6 +163,11 @@ _fabricvaspaceUnbindInstBlk
     KernelGmmu *pKernelGmmu = GPU_GET_KERNEL_GMMU(pGpu);
     INST_BLK_INIT_PARAMS instblkParams = {0};
 
+    if (!pKernelBus->flaInfo.bToggleBindPoint)
+    {
+        return;
+    }
+
     //
     // Check if there are any pending allocations for the fabric vaspace.
     // If there are pending allocations, skip restore and return NV_OK.
@@ -346,10 +358,10 @@ fabricvaspaceAlloc_IMPL
 
     //
     // Allocate VA space of the size and alignment requested.
-    // RM_PAGE_SIZE_HUGE is passed since FLA->PA page size is 2MB.
+    // RM_PAGE_SIZE_HUGE is passed since FLA->PA page size is 2MB or 512MB.
     //
     status = vaspaceAlloc(pFabricVAS->pGVAS, size, align, rangeLo, rangeHi,
-                          RM_PAGE_SIZE_HUGE, flags, pAddr);
+                          RM_PAGE_SIZE_HUGE | RM_PAGE_SHIFT_512M, flags, pAddr);
     if (status != NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR, "Failed to allocate vaspace\n");
@@ -359,6 +371,9 @@ fabricvaspaceAlloc_IMPL
     // Assert that the address returned is pageSize aligned
     NV_ASSERT(NV_IS_ALIGNED64(*pAddr, pageSize));
 
+    pFabricVAS->ucFabricFreeSize  -= size;
+    pFabricVAS->ucFabricInUseSize += size; 
+
     return NV_OK;
 
 failed:
@@ -456,6 +471,11 @@ fabricvaspaceAllocNonContiguous_IMPL
             (*ppAddr)[0] = addr;
             *pNumAddr    = 1;
         }
+        else if (flags.bForceContig)
+        {
+            NV_PRINTF(LEVEL_ERROR, "Failed to allocate contig vaspace\n");
+            goto failed;
+        }
     }
 
     //
@@ -485,6 +505,9 @@ fabricvaspaceAllocNonContiguous_IMPL
         }
     }
 
+    pFabricVAS->ucFabricFreeSize  -= size;
+    pFabricVAS->ucFabricInUseSize += size;
+
     return NV_OK;
 
 failed:
@@ -507,10 +530,15 @@ fabricvaspaceFree_IMPL
     OBJVASPACE *pVAS = staticCast(pFabricVAS, OBJVASPACE);
     OBJGPU     *pGpu = gpumgrGetGpu(gpumgrGetDefaultPrimaryGpu(pVAS->gpuMask));
     KernelBus  *pKernelBus = GPU_GET_KERNEL_BUS(pGpu);
+    NvU64       blockSize;
+    NvBool      bUcFla;
 
     NV_ASSERT_OR_RETURN(pFabricVAS->pGVAS != NULL, NV_ERR_OBJECT_NOT_FOUND);
 
-    NV_ASSERT(vaspaceFree(pFabricVAS->pGVAS, vAddr) == NV_OK);
+    bUcFla = (vAddr >= fabricvaspaceGetUCFlaStart(pFabricVAS) &&
+              vAddr < fabricvaspaceGetUCFlaLimit(pFabricVAS));
+
+    NV_ASSERT(vaspaceFreeV2(pFabricVAS->pGVAS, vAddr, &blockSize) == NV_OK);
 
     kbusFlush_HAL(pGpu, pKernelBus, (BUS_FLUSH_VIDEO_MEMORY |
                                      BUS_FLUSH_SYSTEM_MEMORY | 
@@ -520,6 +548,12 @@ fabricvaspaceFree_IMPL
 
     _fabricvaspaceUnbindInstBlk(pFabricVAS);
 
+    if (bUcFla)
+    {
+        pFabricVAS->ucFabricFreeSize  += blockSize;
+        pFabricVAS->ucFabricInUseSize -= blockSize;
+    }
+
     return NV_OK;
 }
 
@@ -606,8 +640,8 @@ fabricvaspaceGetFreeHeap_IMPL
     NV_ASSERT_OR_RETURN(pFabricVAS->pGVAS != NULL, NV_ERR_OBJECT_NOT_FOUND);
     NV_ASSERT_OR_RETURN(freeSize != NULL,         NV_ERR_INVALID_ARGUMENT);
 
-    return gvaspaceGetFreeHeap(dynamicCast(pFabricVAS->pGVAS, OBJGVASPACE),
-                               freeSize);
+    *freeSize = pFabricVAS->ucFabricFreeSize;
+    return NV_OK;
 }
 
 void
@@ -622,15 +656,24 @@ fabricvaspaceBatchFree_IMPL
     OBJVASPACE *pVAS = staticCast(pFabricVAS, OBJVASPACE);
     OBJGPU     *pGpu = gpumgrGetGpu(gpumgrGetDefaultPrimaryGpu(pVAS->gpuMask));
     KernelBus  *pKernelBus = GPU_GET_KERNEL_BUS(pGpu);
-
+    NvU64       totalFreeSize = 0;
+    NvU64       freeSize;
     NvU32 count = 0;
     NvU32 idx   = 0;
+    NvBool bUcFla;
+
 
     for (count = 0; count < numAddr; count++)
     {
-        NV_ASSERT(vaspaceFree(pFabricVAS->pGVAS, pAddr[idx]) == NV_OK);
+        bUcFla = (pAddr[idx] >= fabricvaspaceGetUCFlaStart(pFabricVAS) &&
+                  pAddr[idx] < fabricvaspaceGetUCFlaLimit(pFabricVAS));
+
+        NV_ASSERT(vaspaceFreeV2(pFabricVAS->pGVAS, pAddr[idx], &freeSize) == NV_OK);
 
         idx += stride;
+
+        if (bUcFla)
+            totalFreeSize += freeSize;
     }
 
     kbusFlush_HAL(pGpu, pKernelBus, (BUS_FLUSH_VIDEO_MEMORY |
@@ -640,6 +683,9 @@ fabricvaspaceBatchFree_IMPL
     fabricvaspaceInvalidateTlb(pFabricVAS, pGpu, PTE_DOWNGRADE);
 
     _fabricvaspaceUnbindInstBlk(pFabricVAS);
+
+    pFabricVAS->ucFabricFreeSize  += totalFreeSize;
+    pFabricVAS->ucFabricInUseSize -= totalFreeSize;
 }
 
 void
@@ -683,8 +729,8 @@ fabricvaspaceGetGpaMemdesc_IMPL
     RmPhysAddr *pteArray = memdescGetPteArray(pRootMemDesc, AT_GPU);
 
     // Check if pteArray[0] is within the VAS range for the mapping GPU.
-    if ((pteArray[0] < vaspaceGetVaStart(staticCast(pFabricVAS, OBJVASPACE))) ||
-        (pteArray[0] > vaspaceGetVaLimit(staticCast(pFabricVAS, OBJVASPACE))))
+    if ((pteArray[0] < fabricvaspaceGetUCFlaStart(pFabricVAS)) ||
+        (pteArray[0] > fabricvaspaceGetUCFlaLimit(pFabricVAS)))
     {
         *ppAdjustedMemdesc = pFabricMemdesc;
         return NV_OK;
@@ -787,3 +833,385 @@ fabricvaspaceVaToGpaMapInsert_IMPL
 
     return NV_OK;
 }
+
+NV_STATUS
+fabricvaspaceAllocMulticast_IMPL
+(
+    FABRIC_VASPACE *pFabricVAS,
+    NvU64           pageSize,
+    NvU64           alignment,
+    VAS_ALLOC_FLAGS flags,
+    NvU64           base,
+    NvU64           size
+)
+{
+    NvU64 rangeLo;
+    NvU64 rangeHi;
+    NvU64 addr = 0;
+    NV_STATUS status;
+
+    NV_ASSERT_OR_RETURN(pFabricVAS->pGVAS != NULL, NV_ERR_OBJECT_NOT_FOUND);
+    NV_ASSERT_OR_RETURN(pageSize >= RM_PAGE_SIZE_HUGE, NV_ERR_INVALID_ARGUMENT);
+    NV_ASSERT_OR_RETURN(alignment != 0, NV_ERR_INVALID_ARGUMENT);
+    NV_ASSERT_OR_RETURN(size != 0, NV_ERR_INVALID_ARGUMENT);
+    NV_ASSERT_OR_RETURN(NV_IS_ALIGNED64(alignment, pageSize), NV_ERR_INVALID_ARGUMENT);
+    NV_ASSERT_OR_RETURN(NV_IS_ALIGNED64(base, pageSize), NV_ERR_INVALID_ARGUMENT);
+    NV_ASSERT_OR_RETURN(NV_IS_ALIGNED64(size, pageSize), NV_ERR_INVALID_ARGUMENT);
+
+    rangeLo = base;
+    rangeHi = base + size - 1;
+
+    status = vaspaceAlloc(pFabricVAS->pGVAS, size, alignment, rangeLo,
+                          rangeHi, pageSize, flags, &addr);
+
+    NV_ASSERT(addr == base);
+
+    return status;
+}
+
+static NV_STATUS
+_fabricVaspaceValidateMapAttrs
+(
+    NvU64  fabricOffset,
+    NvU64  fabricAllocSize,
+    NvU32  fabricPageSize,
+    NvU64  physMapOffset,
+    NvU64  physMapLength,
+    NvU64  physAllocSize,
+    NvU32  physPageSize
+)
+{
+    // Fabric mem offset should be at least phys page size aligned.
+    if (!NV_IS_ALIGNED64(fabricOffset, physPageSize) ||
+        (fabricOffset >= fabricAllocSize))
+    {
+        NV_PRINTF(LEVEL_ERROR,
+                  "Invalid offset passed for the fabric handle\n");
+
+        return NV_ERR_INVALID_OFFSET;
+    }
+
+    if (!NV_IS_ALIGNED64(physMapOffset, physPageSize) ||
+        (physMapOffset >= physAllocSize))
+    {
+        NV_PRINTF(LEVEL_ERROR,
+                  "Invalid offset passed for the physmem handle\n");
+
+        return NV_ERR_INVALID_OFFSET;
+    }
+
+    if ((physMapLength == 0) ||
+        (!NV_IS_ALIGNED64(physMapLength, physPageSize))   ||
+        (physMapLength > (physAllocSize - physMapOffset)) ||
+        (physMapLength > (fabricAllocSize - fabricOffset)))
+    {
+        NV_PRINTF(LEVEL_ERROR,
+                  "Invalid map length passed for the physmem handle\n");
+
+        return NV_ERR_INVALID_ARGUMENT;
+    }
+
+    return NV_OK;
+}
+
+typedef struct FABRIC_VASPACE_MAPPING_REGION
+{
+    NvU64  offset;
+    NvU64  length;
+} FABRIC_VASPACE_MAPPING_REGION;
+
+//
+// In worst case, we can have three regions to map. Two partially filled fabric
+// pages and one (or more) fully filled fabric page(s).
+//
+#define FABRIC_VASPACE_MAPPING_REGIONS_MAX 3
+
+typedef struct FABRIC_VASPACE_MAPPING_REGIONS
+{
+    FABRIC_VASPACE_MAPPING_REGION r[FABRIC_VASPACE_MAPPING_REGIONS_MAX];
+} FABRIC_VASPACE_MAPPING_REGIONS;
+
+static void
+_fabricvaspaceGetMappingRegions
+(
+    NvU64                           fabricOffset,
+    NvU64                           fabricPageSize,
+    NvU64                           physMapLength,
+    FABRIC_VASPACE_MAPPING_REGIONS *pRegions,
+    NvU32                          *pNumRegions
+)
+{
+    NvU64 fabricOffsetAligned = NV_ALIGN_UP64(fabricOffset, fabricPageSize);
+    NvU64 mapLengthAligned = NV_ALIGN_DOWN64(physMapLength, fabricPageSize);
+
+    *pNumRegions = 0;
+
+    if ((fabricOffset < fabricOffsetAligned) &&
+        (physMapLength >= (fabricOffsetAligned - fabricOffset)))
+    {
+        pRegions->r[*pNumRegions].offset = fabricOffset;
+        pRegions->r[*pNumRegions].length = fabricOffsetAligned - fabricOffset;
+
+        fabricOffset += pRegions->r[*pNumRegions].length;
+        physMapLength -= pRegions->r[*pNumRegions].length;
+        mapLengthAligned = NV_ALIGN_DOWN64(physMapLength, fabricPageSize);
+
+        (*pNumRegions)++;
+    }
+
+    if (physMapLength == 0)
+        return;
+
+    if ((fabricOffset == fabricOffsetAligned) &&
+        (mapLengthAligned >= fabricPageSize))
+    {
+        pRegions->r[*pNumRegions].offset = fabricOffset;
+        pRegions->r[*pNumRegions].length = mapLengthAligned;
+
+        fabricOffset += pRegions->r[*pNumRegions].length;
+        physMapLength -= pRegions->r[*pNumRegions].length;
+
+        (*pNumRegions)++;
+    }
+
+    if (physMapLength == 0)
+        return;
+
+    pRegions->r[*pNumRegions].offset = fabricOffset;
+    pRegions->r[*pNumRegions].length = physMapLength;
+
+    (*pNumRegions)++;
+}
+
+void
+fabricvaspaceUnmapPhysMemdesc_IMPL
+(
+    FABRIC_VASPACE    *pFabricVAS,
+    MEMORY_DESCRIPTOR *pFabricMemDesc,
+    NvU64              fabricOffset,
+    MEMORY_DESCRIPTOR *pPhysMemDesc,
+    NvU64              physMapLength
+)
+{
+    OBJGPU *pGpu = pPhysMemDesc->pGpu;
+    NvU32 fabricPageCount;
+    NvU64 fabricAddr;
+    NvU64 fabricPageSize;
+    NvU32 i, j;
+    NvU64 mapLength;
+    FABRIC_VASPACE_MAPPING_REGIONS regions;
+    NvU32 numRegions;
+
+    fabricPageSize = memdescGetPageSize(pFabricMemDesc, AT_GPU);
+
+    NV_ASSERT_OR_RETURN_VOID(dynamicCast(pGpu->pFabricVAS, FABRIC_VASPACE) == \
+                             pFabricVAS);
+
+    _fabricvaspaceGetMappingRegions(fabricOffset, fabricPageSize, physMapLength,
+                                    &regions, &numRegions);
+    NV_ASSERT_OR_RETURN_VOID(numRegions != 0);
+
+    for (i = 0; i < numRegions; i++)
+    {
+        fabricPageCount = ((memdescGetPteArraySize(pFabricMemDesc, AT_GPU) == 1) ||
+                           (regions.r[i].length < fabricPageSize)) ? \
+                          1 : (regions.r[i].length / fabricPageSize);
+        mapLength = (fabricPageCount == 1) ? regions.r[i].length : fabricPageSize;
+        fabricOffset = regions.r[i].offset;
+
+        for (j = 0; j < fabricPageCount; j++)
+        {
+            if (fabricPageCount == 1)
+            {
+                fabricAddr = pFabricMemDesc->_pteArray[0] + fabricOffset;
+            }
+            else
+            {
+                fabricAddr = pFabricMemDesc->_pteArray[fabricOffset / pFabricMemDesc->pageArrayGranularity];
+            }
+
+            vaspaceUnmap(pFabricVAS->pGVAS, pPhysMemDesc->pGpu, fabricAddr, \
+                         fabricAddr + mapLength - 1);
+
+            fabricOffset = fabricOffset + mapLength;
+        }
+    }
+
+    fabricvaspaceInvalidateTlb(pFabricVAS, pPhysMemDesc->pGpu, PTE_DOWNGRADE);
+}
+
+NV_STATUS
+fabricvaspaceMapPhysMemdesc_IMPL
+(
+    FABRIC_VASPACE    *pFabricVAS,
+    MEMORY_DESCRIPTOR *pFabricMemDesc,
+    NvU64              fabricOffset,
+    MEMORY_DESCRIPTOR *pPhysMemDesc,
+    NvU64              physOffset,
+    NvU64              physMapLength,
+    NvU32              flags
+)
+{
+    OBJGPU *pGpu = pPhysMemDesc->pGpu;
+    VirtMemAllocator *pDma = GPU_GET_DMA(pGpu);
+    MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
+    NV_STATUS status;
+    DMA_PAGE_ARRAY pageArray;
+    NvU32 kind;
+    COMPR_INFO comprInfo;
+    NvU32 mapFlags = DMA_UPDATE_VASPACE_FLAGS_UPDATE_ALL |
+                     DMA_UPDATE_VASPACE_FLAGS_SKIP_4K_PTE_CHECK;
+    NvU32 fabricPageCount;
+    NvU64 fabricAddr;
+    NvU32 physPageSize;
+    NvU64 fabricPageSize;
+    NvU64 physAddr;
+    NvU32 i, j;
+    NvU64 mapLength;
+    NvBool bReadOnly = !!(flags & FABRIC_VASPACE_MAP_FLAGS_READ_ONLY);
+    FABRIC_VASPACE_MAPPING_REGIONS regions;
+    NvU32 numRegions;
+    MEMORY_DESCRIPTOR *pTempMemdesc;
+
+    NV_ASSERT_OR_RETURN(pFabricMemDesc != NULL, NV_ERR_INVALID_ARGUMENT);
+    NV_ASSERT_OR_RETURN(pPhysMemDesc != NULL,   NV_ERR_INVALID_ARGUMENT);
+
+    mapFlags |= bReadOnly ? DMA_UPDATE_VASPACE_FLAGS_READ_ONLY : 0;
+
+    NV_ASSERT_OR_RETURN(dynamicCast(pGpu->pFabricVAS, FABRIC_VASPACE) == pFabricVAS,
+                        NV_ERR_INVALID_ARGUMENT);
+
+    physPageSize = memdescGetPageSize(pPhysMemDesc, AT_GPU);
+    fabricPageSize = memdescGetPageSize(pFabricMemDesc, AT_GPU);
+
+    status = _fabricVaspaceValidateMapAttrs(fabricOffset,
+                                            memdescGetSize(pFabricMemDesc),
+                                            fabricPageSize,
+                                            physOffset,
+                                            physMapLength,
+                                            memdescGetSize(pPhysMemDesc),
+                                            physPageSize);
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR, status);
+
+    if (pFabricVAS->bRpcAlloc)
+        return NV_OK;
+
+    status = memmgrGetKindComprFromMemDesc(pMemoryManager, pPhysMemDesc,
+                                           physOffset, &kind, &comprInfo);
+    NV_ASSERT_OK_OR_RETURN(status);
+
+    _fabricvaspaceGetMappingRegions(fabricOffset, fabricPageSize, physMapLength,
+                                    &regions, &numRegions);
+    NV_ASSERT_OR_RETURN(numRegions != 0, NV_ERR_INVALID_ARGUMENT);
+
+    for (i = 0; i < numRegions; i++)
+    {
+        fabricPageCount = ((memdescGetPteArraySize(pFabricMemDesc, AT_GPU) == 1) ||
+                           (regions.r[i].length < fabricPageSize)) ? \
+                          1 : (regions.r[i].length / fabricPageSize);
+        mapLength = (fabricPageCount == 1) ? regions.r[i].length : fabricPageSize;
+        fabricOffset = regions.r[i].offset;
+
+        portMemSet(&pageArray, 0, sizeof(DMA_PAGE_ARRAY));
+        pageArray.count = (memdescGetPteArraySize(pPhysMemDesc, AT_GPU) == 1) ? \
+                          1 : (mapLength / pPhysMemDesc->pageArrayGranularity);
+
+        for (j = 0; j < fabricPageCount; j++)
+        {
+            if (fabricPageCount == 1)
+            {
+                fabricAddr = pFabricMemDesc->_pteArray[0] + fabricOffset;
+            }
+            else
+            {
+                fabricAddr = pFabricMemDesc->_pteArray[fabricOffset / pFabricMemDesc->pageArrayGranularity];
+            }
+
+            if (pageArray.count == 1)
+            {
+                physAddr = pPhysMemDesc->_pteArray[0] + physOffset;
+                pageArray.pData = &physAddr;
+            }
+            else
+            {
+                pageArray.pData = &pPhysMemDesc->_pteArray[physOffset / pPhysMemDesc->pageArrayGranularity];
+            }
+
+            //
+            // When physPageSize is greater than fabricPageSize, to avoid fabric
+            // VAs getting aligned using physPageSize by dmaUpdateVASpace_HAL,
+            // create a tempMemdesc and override its pageSize.
+            //
+            if (fabricPageSize < physPageSize)
+            {
+                status = memdescCreateSubMem(&pTempMemdesc, pPhysMemDesc,
+                                             pPhysMemDesc->pGpu,
+                                             physOffset, mapLength);
+                if (status != NV_OK)
+                    goto fail;
+
+                memdescSetPageSize(pTempMemdesc, AT_GPU, fabricPageSize);
+            }
+            else
+            {
+                pTempMemdesc = pPhysMemDesc;
+            }
+
+            // Map the memory fabric object at the given physical memory offset.
+            status = dmaUpdateVASpace_HAL(pGpu, pDma, pFabricVAS->pGVAS, pTempMemdesc,
+                                      NULL, fabricAddr, fabricAddr + mapLength - 1,
+                                      mapFlags, &pageArray, 0, &comprInfo, 0,
+                                      NV_MMU_PTE_VALID_TRUE,
+                                      NV_MMU_PTE_APERTURE_VIDEO_MEMORY,
+                                      BUS_INVALID_PEER, NVLINK_INVALID_FABRIC_ADDR,
+                                      DMA_DEFER_TLB_INVALIDATE, NV_FALSE, fabricPageSize);
+
+            if (pTempMemdesc != pPhysMemDesc)
+                memdescDestroy(pTempMemdesc);
+
+            if (status != NV_OK)
+                goto fail;
+
+            physOffset = physOffset + mapLength;
+            fabricOffset = fabricOffset + mapLength;
+        }
+    }
+
+    fabricvaspaceInvalidateTlb(pFabricVAS, pPhysMemDesc->pGpu, PTE_UPGRADE);
+
+    return NV_OK;
+
+fail:
+    for (j = 0; j < i; j++)
+        fabricvaspaceUnmapPhysMemdesc(pFabricVAS, pFabricMemDesc,
+                                      regions.r[j].offset, pPhysMemDesc,
+                                      regions.r[j].length);
+
+    return status;
+}
+
+NV_STATUS
+fabricvaspaceInitUCRange_IMPL
+(
+    FABRIC_VASPACE *pFabricVAS,
+    OBJGPU         *pGpu,
+    NvU64           fabricBase,
+    NvU64           fabricSize
+)
+{
+    if (fabricvaspaceGetUCFlaLimit(pFabricVAS) != 0)
+        return NV_ERR_IN_USE;
+
+    if (fabricSize != 0)
+    {
+        NV_PRINTF(LEVEL_INFO, "Setting UC Base: %llx, size: %llx \n",
+                  fabricBase, fabricSize);
+        pFabricVAS->ucFabricBase  = fabricBase;
+        pFabricVAS->ucFabricLimit = fabricBase + fabricSize - 1;
+        pFabricVAS->ucFabricInUseSize = 0;
+        pFabricVAS->ucFabricFreeSize = fabricSize;
+    }
+
+    return NV_OK;
+}
+
