@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2018-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2018-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -69,7 +69,7 @@ memConstruct_IMPL
         pMemory->pDevice = dynamicCast(pGrandParentRef->pResource, Device);
 
         if (pMemory->pDevice == NULL)
-                return NV_ERR_INVALID_OBJECT_HANDLE;
+            return NV_ERR_INVALID_OBJECT_HANDLE;
     }
 
     // If child of device, we have a pGpu
@@ -78,6 +78,8 @@ memConstruct_IMPL
         // NOTE: pGpu and pDevice be NULL for NoDeviceMemory
         pMemory->pGpu = CliGetGpuFromContext(pResourceRef, &pMemory->bBcResource);
 
+        NV_ASSERT_OR_RETURN(pMemory->pGpu != NULL, NV_ERR_INVALID_ARGUMENT);
+
         // Set thread BC state
         gpuSetThreadBcState(pMemory->pGpu, pMemory->bBcResource);
     }
@@ -254,7 +256,7 @@ memCreateKernelMapping_IMPL
 {
     NV_STATUS status;
 
-    NV_CHECK_OK_OR_RETURN(LEVEL_INFO, memIsReady(pMemory));
+    NV_CHECK_OK_OR_RETURN(LEVEL_INFO, memIsReady(pMemory, NV_FALSE));
 
     if (pMemory->KernelVAddr == NvP64_NULL)
     {
@@ -476,7 +478,7 @@ memConstructCommon_IMPL
 done:
     if (status != NV_OK)
     {
-        if (pMemory != NULL && pMemory->pHwResource != NULL)
+        if (pMemory->pHwResource != NULL)
         {
             portMemFree(pMemory->pHwResource);
         }
@@ -489,6 +491,21 @@ done:
     return status;
 }
 
+static NvBool
+_memCheckHostVgpuDeviceExists
+(
+    OBJGPU *pGpu
+)
+{
+    NV_STATUS status;
+
+    KERNEL_HOST_VGPU_DEVICE *pKernelHostVgpuDevice = NULL;
+
+    NV_ASSERT_OK_OR_ELSE(status, vgpuGetCallingContextKernelHostVgpuDevice(pGpu, &pKernelHostVgpuDevice), return NV_FALSE);
+
+    return (pKernelHostVgpuDevice != NULL);
+}
+
 static void
 _memDestructCommonWithDevice
 (
@@ -542,7 +559,7 @@ _memDestructCommonWithDevice
         if (--pMemory->pHwResource->refCount == 0)
         {
             MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
-            NvBool bHostVgpuDeviceExists = NV_FALSE;
+            NvBool bHostVgpuDeviceExists = _memCheckHostVgpuDeviceExists(pGpu);
 
             if ((pMemory->categoryClassId == NV01_MEMORY_SYSTEM && memmgrComprSupported(pMemoryManager, ADDR_SYSMEM)) ||
                 (bHostVgpuDeviceExists && (pMemory->pHwResource->isGuestAllocated)))
@@ -693,7 +710,7 @@ memGetByHandle_IMPL
     if (*ppMemory == NULL)
         return NV_ERR_INVALID_OBJECT_HANDLE;
 
-    NV_CHECK_OK_OR_RETURN(LEVEL_INFO, memIsReady(*ppMemory));
+    NV_CHECK_OK_OR_RETURN(LEVEL_INFO, memIsReady(*ppMemory, NV_FALSE));
 
     return NV_OK;
 }
@@ -721,7 +738,8 @@ memGetByHandleAndGroupedGpu_IMPL
 NV_STATUS
 memIsReady_IMPL
 (
-    Memory *pMemory
+    Memory *pMemory,
+    NvBool  bCopyConstructorContext
 )
 {
     if (pMemory->pMemDesc == NULL)
@@ -740,7 +758,7 @@ memControl_IMPL
 {
     RmCtrlParams *pRmCtrlParams = pParams->pLegacyParams;
 
-    NV_CHECK_OK_OR_RETURN(LEVEL_INFO, memIsReady(pMemory));
+    NV_CHECK_OK_OR_RETURN(LEVEL_INFO, memIsReady(pMemory, NV_FALSE));
 
     if (!pMemory->pGpu)
         return NV_ERR_INVALID_OBJECT_PARENT;
@@ -787,7 +805,7 @@ memCopyConstruct_IMPL
     NV_ASSERT_OR_RETURN(pDstParentRef != NULL, NV_ERR_INVALID_OBJECT_PARENT);
     NV_ASSERT_OR_RETURN(pMemorySrc != NULL, NV_ERR_INVALID_OBJECT_HANDLE);
 
-    NV_CHECK_OK_OR_RETURN(LEVEL_INFO, memIsReady(pMemorySrc));
+    NV_CHECK_OK_OR_RETURN(LEVEL_INFO, memIsReady(pMemorySrc, NV_TRUE));
 
     //
     // Must return early when parent is Client.
@@ -945,7 +963,7 @@ memGetMemInterMapParams_IMPL
     NvHandle            hMemoryDevice = 0;
     OBJGPU             *pSrcGpu = pGpu;
 
-    NV_CHECK_OK_OR_RETURN(LEVEL_INFO, memIsReady(pMemory));
+    NV_CHECK_OK_OR_RETURN(LEVEL_INFO, memIsReady(pMemory, NV_FALSE));
 
     if (pMemoryRef->pParentRef != NULL)
     {
@@ -1014,7 +1032,7 @@ memGetMemoryMappingDescriptor_IMPL
     MEMORY_DESCRIPTOR **ppMemDesc
 )
 {
-    NV_CHECK_OK_OR_RETURN(LEVEL_INFO, memIsReady(pMemory));
+    NV_CHECK_OK_OR_RETURN(LEVEL_INFO, memIsReady(pMemory, NV_FALSE));
     if (pMemory->pGpu != NULL)
     {
         *ppMemDesc = memdescGetMemDescFromGpu(pMemory->pMemDesc, pMemory->pGpu);
@@ -1026,3 +1044,29 @@ memGetMemoryMappingDescriptor_IMPL
     return NV_OK;
 }
 
+NV_STATUS
+memIsDuplicate_IMPL
+(
+    Memory   *pMemory,
+    NvHandle  hMemory,
+    NvBool   *pDuplicate
+)
+{
+    RsClient *pClient = RES_GET_CLIENT(pMemory);
+    Memory *pMemory1;
+
+    NV_CHECK_OK_OR_RETURN(LEVEL_SILENT,
+                          memIsReady(pMemory, NV_FALSE));
+
+    NV_CHECK_OK_OR_RETURN(LEVEL_SILENT,
+                          memGetByHandle(pClient, hMemory, &pMemory1));
+
+    //
+    // Do not dereference pMemdesc here. We only take RMAPI RO lock and
+    // client lock in this context.
+    //
+
+    *pDuplicate = (pMemory->pMemDesc == pMemory1->pMemDesc);
+
+    return NV_OK;
+}
