@@ -32,6 +32,7 @@
 #include "core/system.h"
 #include "os/os.h"
 #include "nvpcie.h"
+#include "nvdevid.h"
 
 #include "nvcst.h"
 
@@ -856,25 +857,6 @@ Intel_0685_setupFunc
     return NV_OK;
 }
 
-// Intel Ice Lake platform
-static NV_STATUS
-Intel_IceLake_setupFunc
-(
-    OBJCL *pCl
-)
-{
-    OBJSYS *pSys = SYS_GET_INSTANCE();
-
-    // Intel IceLake
-    if ((pSys->cpuInfo.family == 0x6) &&
-        (pSys->cpuInfo.model == 0x6a) &&
-        (pSys->cpuInfo.stepping == 0x6))
-    {
-        pCl->setProperty(pCl, PDB_PROP_CL_BUG_3562968_WAR_ALLOW_PCIE_ATOMICS, NV_TRUE);
-    }
-    return NV_OK;
-}
-
 // Intel Z590 platform (Rocket Lake)
 static NV_STATUS
 Intel_4381_setupFunc
@@ -1117,15 +1099,6 @@ AMD_X370_setupFunc
     OBJCL *pCl
 )
 {
-    OBJSYS *pSys = SYS_GET_INSTANCE();
-
-    // AMD Milan
-    if (pSys->cpuInfo.family == 0x19 &&
-        pSys->cpuInfo.model == 0x1 &&
-        pSys->cpuInfo.stepping == 0x1)
-    {
-        pCl->setProperty(pCl, PDB_PROP_CL_BUG_3562968_WAR_ALLOW_PCIE_ATOMICS, NV_TRUE);
-    }
 
     // Set ASPM L0S\L1 properties
     _Set_ASPM_L0S_L1(pCl, NV_FALSE, NV_FALSE);
@@ -1261,18 +1234,6 @@ Fujitsu_A64FX_setupFunc
     return NV_OK;
 }
 
-// Phytium FT2000 Setup Function
-static NV_STATUS
-Phytium_FT2000_setupFunc
-(
-    OBJCL *pCl
-)
-{
-    // TODO Need to check if any more PDB properties should be set
-    pCl->setProperty(pCl, PDB_PROP_CL_IS_CHIPSET_IO_COHERENT, NV_TRUE);
-    return NV_OK;
-}
-
 // Ampere Quicksilver Setup Function
 static NV_STATUS
 Ampere_Altra_setupFunc
