@@ -28,12 +28,14 @@
 #include "diagnostics/tracer.h"
 #include "core/locks.h"
 #include "core/thread_state.h"
+#include "virtualization/hypervisor/hypervisor.h"
 #include "gpu/device/device.h"
 
 #include "entry_points.h"
 #include "resserv/rs_access_map.h"
 #include "gpu/gpu.h"
 #include "gpu/subdevice/subdevice.h"
+#include "rmapi/rmapi_utils.h"
 
 #include "ctrl/ctrl0000/ctrl0000client.h" // NV0000_CTRL_CMD_CLIENT_*
 #include "ctrl/ctrl0000/ctrl0000gpu.h" // NV0000_CTRL_CMD_GPU_*
@@ -46,6 +48,7 @@
 #include "ctrl/ctrlc370/ctrlc370chnc.h" // NVC370_CTRL_CMD_*
 #include "ctrl/ctrl9010.h" //NV9010_CTRL_CMD_SET_VBLANK_NOTIFICATION
 #include "ctrl/ctrl2080/ctrl2080tmr.h" // NV2080_CTRL_CMD_TIMER_*
+#include "ctrl/ctrl0000/ctrl0000gpuacct.h" // NV0000_CTRL_CMD_GPUACCT_*
 
 static NV_STATUS
 releaseDeferRmCtrlBuffer(RmCtrlDeferredCmd* pRmCtrlDeferredCmd)
@@ -205,7 +208,7 @@ _rmControlDeferred(RmCtrlParams *pRmCtrlParams, NvP64 pUserParams, NvU32 paramsS
                         // flag is set and after this rmctrl failed to acquire lock.
 
                         // LOCK: try to acquire GPUs lock
-                        if (rmGpuLocksAcquire(GPUS_LOCK_FLAGS_COND_ACQUIRE,
+                        if (rmGpuLocksAcquire(GPU_LOCK_FLAGS_COND_ACQUIRE,
                                               RM_LOCK_MODULES_CLIENT) == NV_OK)
                         {
                             if (osCondAcquireRmSema(pSys->pSema) == NV_OK)
@@ -257,7 +260,10 @@ serverControlApiCopyIn
     pUserParams = NV_PTR_TO_NvP64(pRmCtrlParams->pParams);
     paramsSize = pRmCtrlParams->paramsSize;
 
-    RMAPI_PARAM_COPY_INIT(*pParamCopy, pRmCtrlParams->pParams, pUserParams, paramsSize, 1);
+    RMAPI_PARAM_COPY_INIT(*pParamCopy, pRmCtrlParams->pParams, pUserParams, 1, paramsSize);
+
+    if (pCookie->apiCopyFlags & RMCTRL_API_COPY_FLAGS_SKIP_COPYIN)
+        pParamCopy->flags |= RMAPI_PARAM_COPY_FLAGS_SKIP_COPYIN;
 
     rmStatus = rmapiParamsAcquire(pParamCopy, (pRmCtrlParams->secInfo.paramLocation == PARAM_LOCATION_USER));
     if (rmStatus != NV_OK)
@@ -289,6 +295,16 @@ serverControlApiCopyOut
     NvBool     bFreeParamCopy;
 
     NV_ASSERT_OR_RETURN(pCookie != NULL, NV_ERR_INVALID_ARGUMENT);
+    NV_ASSERT_OR_RETURN(pRmCtrlParams != NULL, NV_ERR_INVALID_ARGUMENT);
+
+    if ((pCookie->apiCopyFlags & RMCTRL_API_COPY_FLAGS_SET_CONTROL_CACHE) && rmStatus == NV_OK)
+    {
+        rmapiControlCacheSet(pRmCtrlParams->hClient,
+                             pRmCtrlParams->hObject,
+                             pRmCtrlParams->cmd,
+                             pRmCtrlParams->pParams,
+                             pRmCtrlParams->paramsSize);
+    }
 
     pParamCopy = &pCookie->paramCopy;
     pEmbeddedParamCopies = pCookie->embeddedParamCopies;
@@ -296,7 +312,9 @@ serverControlApiCopyOut
     bFreeParamCopy = pCookie->bFreeParamCopy;
     bFreeEmbeddedCopy = pCookie->bFreeEmbeddedCopy;
 
-    if ((rmStatus != NV_OK) && !(pCookie->ctrlFlags & RMCTRL_FLAGS_COPYOUT_ON_ERROR))
+    if ((rmStatus != NV_OK) &&
+        (!(pCookie->ctrlFlags & RMCTRL_FLAGS_COPYOUT_ON_ERROR) ||
+        (pCookie->apiCopyFlags & RMCTRL_API_COPY_FLAGS_FORCE_SKIP_COPYOUT_ON_ERROR)))
     {
         pParamCopy->flags |= RMAPI_PARAM_COPY_FLAGS_SKIP_COPYOUT;
 
@@ -526,7 +544,7 @@ _rmapiRmControl(NvHandle hClient, NvHandle hObject, NvU32 cmd, NvP64 pUserParams
         // LOCK: try to acquire GPUs lock
         if (osCondAcquireRmSema(pSys->pSema) == NV_OK)
         {
-            if (rmGpuLocksAcquire(GPUS_LOCK_FLAGS_COND_ACQUIRE, RM_LOCK_MODULES_CLIENT) == NV_OK)
+            if (rmGpuLocksAcquire(GPU_LOCK_FLAGS_COND_ACQUIRE, RM_LOCK_MODULES_CLIENT) == NV_OK)
             {
                 lockInfo.state |= RM_LOCK_STATES_GPUS_LOCK_ACQUIRED;
                 lockInfo.flags |= RM_LOCK_FLAGS_NO_API_LOCK |
@@ -535,7 +553,7 @@ _rmapiRmControl(NvHandle hClient, NvHandle hObject, NvU32 cmd, NvP64 pUserParams
                 rmStatus = serverControl(&g_resServ, &rmCtrlParams);
 
                 // UNLOCK: release GPUs lock
-                rmGpuLocksRelease(GPUS_LOCK_FLAGS_COND_ACQUIRE, osIsISR() ? rmCtrlParams.pGpu : NULL);
+                rmGpuLocksRelease(GPU_LOCK_FLAGS_COND_ACQUIRE, osIsISR() ? rmCtrlParams.pGpu : NULL);
             }
             else
             {
@@ -556,6 +574,27 @@ _rmapiRmControl(NvHandle hClient, NvHandle hObject, NvU32 cmd, NvP64 pUserParams
         // Normal rmctrl request.
         //
 
+        if (rmapiCmdIsCacheable(cmd, NV_FALSE))
+        {
+            rmCtrlParams.pCookie->apiCopyFlags |= RMCTRL_API_COPY_FLAGS_SKIP_COPYIN;
+            rmCtrlParams.pCookie->apiCopyFlags |= RMCTRL_API_COPY_FLAGS_FORCE_SKIP_COPYOUT_ON_ERROR;
+
+            serverControlApiCopyIn(&g_resServ, &rmCtrlParams, rmCtrlParams.pCookie);
+            rmStatus = rmapiControlCacheGet(hClient, hObject, cmd, rmCtrlParams.pParams, paramsSize);
+            serverControlApiCopyOut(&g_resServ, &rmCtrlParams, rmCtrlParams.pCookie, rmStatus);
+
+            if (rmStatus == NV_OK)
+            {
+                goto done;
+            }
+            else
+            {
+                // reset cookie if cache get failed
+                portMemSet(rmCtrlParams.pCookie, 0, sizeof(RS_CONTROL_COOKIE));
+                rmCtrlParams.pCookie->apiCopyFlags |= RMCTRL_API_COPY_FLAGS_SET_CONTROL_CACHE;
+            }
+        }
+
         RM_API_CONTEXT rmApiContext = {0};
         rmStatus = rmapiPrologue(pRmApi, &rmApiContext);
         if (rmStatus != NV_OK)
@@ -571,6 +610,47 @@ done:
     return rmStatus;
 }
 
+static NvBool
+serverControl_ValidateVgpu
+(
+    OBJGPU *pGpu,
+    NvU32 cmd,
+    RS_PRIV_LEVEL privLevel,
+    const NvU32 cookieFlags
+)
+{
+    NvBool bPermissionGranted = NV_FALSE;
+
+    // Check if context is already sufficiently admin privileged
+    if (cookieFlags & RMCTRL_FLAGS_PRIVILEGED)
+    {
+        if (privLevel >= RS_PRIV_LEVEL_USER_ROOT)
+        {
+            bPermissionGranted = NV_TRUE;
+        }
+    }
+
+    //
+    // If context is not privileged enough, check whether this
+    // control call is allowed in current hypervisor environment
+    //
+    if (!bPermissionGranted)
+    {
+        {
+            // For non-NV0000, identify current hypervisor environment and check for allow flag
+            if
+            (
+                (IS_SRIOV_WITH_VGPU_GSP_ENABLED(pGpu) && (cookieFlags & RMCTRL_FLAGS_CPU_PLUGIN_FOR_VGPU_GSP))
+            )
+            {
+                bPermissionGranted = NV_TRUE;
+            }
+        }
+    }
+
+    return bPermissionGranted;
+}
+
 // validate rmctrl flags
 NV_STATUS serverControl_ValidateCookie
 (
@@ -578,7 +658,25 @@ NV_STATUS serverControl_ValidateCookie
     RS_CONTROL_COOKIE *pRmCtrlExecuteCookie
 )
 {
-    NV_STATUS           status;
+    NV_STATUS status;
+    OBJGPU *pGpu;
+    CALL_CONTEXT *pCallContext = resservGetTlsCallContext();
+
+    if (pCallContext == NULL)
+    {
+        NV_PRINTF(LEVEL_ERROR, "Calling context is NULL!\n");
+        return NV_ERR_INVALID_PARAMETER;
+    }
+
+    if (RMCFG_FEATURE_PLATFORM_GSP)
+    {
+        pGpu = gpumgrGetSomeGpu();
+        if (pGpu == NULL)
+        {
+            NV_PRINTF(LEVEL_ERROR, "GPU is not found\n");
+            return NV_ERR_INVALID_STATE;
+        }
+    }
 
     if (g_resServ.bRsAccessEnabled)
     {
@@ -621,34 +719,68 @@ NV_STATUS serverControl_ValidateCookie
             return NV_ERR_NOT_SUPPORTED;
     }
 
-    if (pRmCtrlExecuteCookie->ctrlFlags & RMCTRL_FLAGS_PRIVILEGED)
+    //
+    // Narrow down usecase as much as possible to CPU-plugin.
+    // Must be running in hypervisor, at least cached privileged, not a kernel context and
+    // accessing a privileged or kernel privileged control call.
+    //
+    if (hypervisorIsVgxHyper() &&
+        clientIsAdmin(pCallContext->pClient, clientGetCachedPrivilege(pCallContext->pClient)) &&
+        (pRmCtrlParams->secInfo.privLevel != RS_PRIV_LEVEL_KERNEL) &&
+        !(pRmCtrlExecuteCookie->ctrlFlags & RMCTRL_FLAGS_NON_PRIVILEGED))
     {
-        //
-        // Calls originating from usermode require admin perms while calls
-        // originating from other kernel drivers are always allowed.
-        //
-        if ((pRmCtrlParams->secInfo.privLevel < RS_PRIV_LEVEL_USER_ROOT)
-           )
+        // VGPU CPU-Plugin (Legacy Non-SRIOV, SRIOV-HYPERV, SRIOV-LEGACY, SRIOV-Offload), and Admin or kernel clients running in hypervisor
+        NvBool bPermissionGranted = serverControl_ValidateVgpu(pRmCtrlParams->pGpu,
+                                                               pRmCtrlParams->cmd,
+                                                               pRmCtrlParams->secInfo.privLevel,
+                                                               pRmCtrlExecuteCookie->ctrlFlags);
+        if (!bPermissionGranted)
         {
             NV_PRINTF(LEVEL_WARNING,
-                      "hClient: 0x%08x, hObject 0x%08x, cmd 0x%08x: non-privileged context issued privileged cmd\n",
+                      "hClient: 0x%08x, hObject 0x%08x, cmd 0x%08x: non-privileged hypervisor context issued privileged cmd\n",
                       pRmCtrlParams->hClient, pRmCtrlParams->hObject,
                       pRmCtrlParams->cmd);
             return NV_ERR_INSUFFICIENT_PERMISSIONS;
         }
     }
-
-    // permissions check for KERNEL_PRIVILEGED (default) unless NON_PRIVILEGED, PRIVILEGED or INTERNAL is specified
-    if ( !(pRmCtrlExecuteCookie->ctrlFlags & (RMCTRL_FLAGS_NON_PRIVILEGED | RMCTRL_FLAGS_PRIVILEGED | RMCTRL_FLAGS_INTERNAL)))
+    else
     {
-        if ((pRmCtrlParams->secInfo.privLevel < RS_PRIV_LEVEL_KERNEL)
-           )
+        //
+        // Non-Hypervisor clients
+        // PF clients
+        // Unprivileged processes running in Hypervisor
+        // Privileged processes running in Hypervisor, executing an unprivileged control call.
+        // Kernel privileged processes running in Hypervisor
+        //
+
+        // permissions check for PRIVILEGED controls
+        if (pRmCtrlExecuteCookie->ctrlFlags & RMCTRL_FLAGS_PRIVILEGED)
         {
-            NV_PRINTF(LEVEL_WARNING,
-                      "hClient: 0x%08x, hObject 0x%08x, cmd 0x%08x: non-kernel client issued kernel-only cmd\n",
-                      pRmCtrlParams->hClient, pRmCtrlParams->hObject,
-                      pRmCtrlParams->cmd);
-            return NV_ERR_INSUFFICIENT_PERMISSIONS;
+            //
+            // Calls originating from usermode require admin perms while calls
+            // originating from other kernel drivers are always allowed.
+            //
+            if (pRmCtrlParams->secInfo.privLevel < RS_PRIV_LEVEL_USER_ROOT)
+            {
+                NV_PRINTF(LEVEL_WARNING,
+                          "hClient: 0x%08x, hObject 0x%08x, cmd 0x%08x: non-privileged context issued privileged cmd\n",
+                          pRmCtrlParams->hClient, pRmCtrlParams->hObject,
+                          pRmCtrlParams->cmd);
+                return NV_ERR_INSUFFICIENT_PERMISSIONS;
+            }
+        }
+
+        // permissions check for KERNEL_PRIVILEGED (default) unless NON_PRIVILEGED, PRIVILEGED or INTERNAL is specified
+        if (!(pRmCtrlExecuteCookie->ctrlFlags & (RMCTRL_FLAGS_NON_PRIVILEGED | RMCTRL_FLAGS_PRIVILEGED | RMCTRL_FLAGS_INTERNAL)))
+        {
+            if (pRmCtrlParams->secInfo.privLevel < RS_PRIV_LEVEL_KERNEL)
+            {
+                NV_PRINTF(LEVEL_WARNING,
+                          "hClient: 0x%08x, hObject 0x%08x, cmd 0x%08x: non-kernel client issued kernel-only cmd\n",
+                          pRmCtrlParams->hClient, pRmCtrlParams->hObject,
+                          pRmCtrlParams->cmd);
+                return NV_ERR_INSUFFICIENT_PERMISSIONS;
+            }
         }
     }
 
@@ -681,6 +813,36 @@ serverControlLookupLockFlags
     LOCK_ACCESS_TYPE *pAccess
 )
 {
+    //
+    // Calls with LOCK_TOP doesn't fill in the cookie param correctly.
+    // This is just a WAR for this. Since the optimisation that depends on this
+    // flag only works in full GSP offload mode, it's gated to that.
+    //
+    NvBool areAllGpusInOffloadMode = gpumgrAreAllGpusInOffloadMode();
+    NvU32 controlFlags = pRmCtrlExecuteCookie->ctrlFlags;
+    if (controlFlags == 0 && !RMCFG_FEATURE_PLATFORM_GSP && areAllGpusInOffloadMode)
+    {
+        NV_STATUS status = rmapiutilGetControlInfo(pRmCtrlParams->cmd, &controlFlags, NULL);
+        if (status != NV_OK)
+        {
+            NV_PRINTF(LEVEL_INFO,
+                      "rmapiutilGetControlInfo(cmd=0x%x, out flags=0x%x, NULL) = status=0x%x\n",
+                      pRmCtrlParams->cmd, controlFlags, status);
+        }
+    }
+
+    //
+    // If the control is ROUTE_TO_PHYSICAL, and we're in GSP offload mode,
+    // we can use a more relaxed locking mode:
+    //    1. Only lock the single device and not all GPUs
+    //    2. Take the API lock for READ instead of WRITE.
+    // Unfortunately, at this point we don't have the pGpu yet to check if it
+    // is in offload mode or not. So, instead, these optimizations are only
+    // done if *all* GPUs in the system are in offload mode.
+    //
+    NvBool bUseGspLockingMode = areAllGpusInOffloadMode &&
+                                (controlFlags & RMCTRL_FLAGS_ROUTE_TO_PHYSICAL);
+
     if (pAccess == NULL)
         return NV_ERR_INVALID_ARGUMENT;
 
@@ -695,7 +857,18 @@ serverControlLookupLockFlags
         }
 
         if (pRmCtrlExecuteCookie->ctrlFlags & RMCTRL_FLAGS_API_LOCK_READONLY)
+        {
             *pAccess = LOCK_ACCESS_READ;
+        }
+
+        //
+        // ROUTE_TO_PHYSICAL controls always take the READ API lock. This only applies
+        // to GSP clients: Only there can we guarantee per-gpu execution of commands.
+        //
+        if (g_resServ.bRouteToPhysicalLockBypass && bUseGspLockingMode)
+        {
+            *pAccess = LOCK_ACCESS_READ;
+        }
 
         return NV_OK;
     }
@@ -719,7 +892,8 @@ serverControlLookupLockFlags
         }
         else
         {
-            if (pRmCtrlExecuteCookie->ctrlFlags & RMCTRL_FLAGS_GPU_LOCK_DEVICE_ONLY)
+            if ((pRmCtrlExecuteCookie->ctrlFlags & RMCTRL_FLAGS_GPU_LOCK_DEVICE_ONLY) ||
+                (g_resServ.bRouteToPhysicalLockBypass && bUseGspLockingMode))
             {
                 pLockInfo->flags |= RM_LOCK_FLAGS_NO_GPUS_LOCK;
                 pLockInfo->flags |= RM_LOCK_FLAGS_GPU_GROUP_LOCK;
