@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2015-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2015-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -34,10 +34,44 @@
 #error "DEBUG module must be present for memory tracking"
 #endif
 
-#if !PORT_IS_MODULE_SUPPORTED(atomic)
+#if PORT_MEM_TRACK_USE_LIMIT
+#include "os/os.h"
+#define PORT_MEM_LIMIT_MAX_PIDS 32
+#endif
+
+#if NVOS_IS_LIBOS
+#define PORT_MEM_THREAD_SAFE_ALLOCATIONS 0
+#else
+#define PORT_MEM_THREAD_SAFE_ALLOCATIONS 1
+#endif
+
+#if PORT_MEM_THREAD_SAFE_ALLOCATIONS && !PORT_IS_MODULE_SUPPORTED(atomic)
 #error "ATOMIC module must be present for memory tracking"
 #endif
 
+#if PORT_MEM_THREAD_SAFE_ALLOCATIONS
+#define PORT_MEM_ATOMIC_ADD_SIZE portAtomicAddSize
+#define PORT_MEM_ATOMIC_SUB_SIZE portAtomicSubSize
+#define PORT_MEM_ATOMIC_DEC_U32 portAtomicDecrementU32
+#define PORT_MEM_ATOMIC_INC_U32 portAtomicIncrementU32
+#define PORT_MEM_ATOMIC_SET_U32 portAtomicSetU32
+#define PORT_MEM_ATOMIC_CAS_SIZE portAtomicCompareAndSwapSize
+#define PORT_MEM_ATOMIC_CAS_U32 portAtomicCompareAndSwapU32
+#else
+//
+// We can just stub out the atomic operations for non-atomic ones and not waste
+// waste cycles on synchronization
+//
+#define PORT_MEM_ATOMIC_ADD_SIZE(pVal, val) (*((NvSPtr *)pVal) += val)
+#define PORT_MEM_ATOMIC_SUB_SIZE(pVal, val) (*((NvSPtr *)pVal) -= val)
+#define PORT_MEM_ATOMIC_DEC_U32(pVal)      (--(*((NvU32 *)pVal)))
+#define PORT_MEM_ATOMIC_INC_U32(pVal)      (++(*((NvU32 *)pVal)))
+#define PORT_MEM_ATOMIC_SET_U32(pVal, val) (*((NvU32 *)pVal) = val)
+#define PORT_MEM_ATOMIC_CAS_SIZE(pVal, newVal, oldVal) \
+    ((*pVal == oldVal) ? ((*((NvSPtr *)pVal) = newVal), NV_TRUE) : NV_FALSE)
+#define PORT_MEM_ATOMIC_CAS_U32(pVal, newVal, oldVal) \
+    ((*pVal == oldVal) ? ((*((NvU32 *)pVal) = newVal), NV_TRUE) : NV_FALSE)
+#endif // !PORT_MEM_THREAD_SAFE_ALLOCATIONS
 
 struct PORT_MEM_ALLOCATOR_IMPL
 {
@@ -60,6 +94,11 @@ struct PORT_MEM_ALLOCATOR_IMPL
 
 // Simple implementation of a spinlock that is going to be used where sync module is not included.
 #if !PORT_IS_MODULE_SUPPORTED(sync)
+
+#if NVCPU_IS_RISCV64
+#error "Sync module should be enabled for RISC-V builds"
+#endif
+
 typedef volatile NvU32 PORT_SPINLOCK;
 static NvLength portSyncSpinlockSize = sizeof(PORT_SPINLOCK);
 static NV_STATUS portSyncSpinlockInitialize(PORT_SPINLOCK *pSpinlock)
@@ -69,11 +108,11 @@ static NV_STATUS portSyncSpinlockInitialize(PORT_SPINLOCK *pSpinlock)
 }
 static void portSyncSpinlockAcquire(PORT_SPINLOCK *pSpinlock)
 {
-    while (!portAtomicCompareAndSwapU32(pSpinlock, 1, 0));
+    while (!PORT_MEM_ATOMIC_CAS_U32(pSpinlock, 1, 0));
 }
 static void portSyncSpinlockRelease(PORT_SPINLOCK *pSpinlock)
 {
-    portAtomicSetU32(pSpinlock, 0);
+    PORT_MEM_ATOMIC_SET_U32(pSpinlock, 0);
 }
 static void portSyncSpinlockDestroy(PORT_SPINLOCK *pSpinlock)
 {
@@ -141,13 +180,12 @@ _portMemCounterInc
     NvU32 activeAllocs;
     NvLength activeSize = 0;
 
-    activeAllocs = portAtomicIncrementU32(&pCounter->activeAllocs);
-    portAtomicIncrementU32(&pCounter->totalAllocs);
-    if (PORT_MEM_TRACK_USE_FENCEPOSTS)
-    {
-        activeSize = portAtomicAddSize(&pCounter->activeSize, size);
-    }
-    portAtomicAddSize(&pCounter->totalSize, size);
+    activeAllocs = PORT_MEM_ATOMIC_INC_U32(&pCounter->activeAllocs);
+    PORT_MEM_ATOMIC_INC_U32(&pCounter->totalAllocs);
+#if PORT_MEM_TRACK_USE_FENCEPOSTS
+    activeSize = PORT_MEM_ATOMIC_ADD_SIZE(&pCounter->activeSize, size);
+#endif
+    PORT_MEM_ATOMIC_ADD_SIZE(&pCounter->totalSize, size);
 
     // Atomically compare the peak value with the active, and update if greater.
     while (1)
@@ -155,14 +193,14 @@ _portMemCounterInc
         NvU32 peakAllocs = pCounter->peakAllocs;
         if (activeAllocs <= peakAllocs)
             break;
-        portAtomicCompareAndSwapU32(&pCounter->peakAllocs, activeAllocs, peakAllocs);
+        PORT_MEM_ATOMIC_CAS_U32(&pCounter->peakAllocs, activeAllocs, peakAllocs);
     }
     while (1)
     {
         NvLength peakSize = pCounter->peakSize;
         if (activeSize <= peakSize)
             break;
-        portAtomicCompareAndSwapSize(&pCounter->peakSize, activeSize, peakSize);
+        PORT_MEM_ATOMIC_CAS_SIZE(&pCounter->peakSize, activeSize, peakSize);
     }
 }
 static NV_INLINE void
@@ -172,12 +210,12 @@ _portMemCounterDec
     void             *pMem
 )
 {
-    portAtomicDecrementU32(&pCounter->activeAllocs);
-    if (PORT_MEM_TRACK_USE_FENCEPOSTS)
-    {
-        portAtomicSubSize(&pCounter->activeSize,
-                          ((PORT_MEM_FENCE_HEAD *)pMem-1)->blockSize);
-    }
+    PORT_UNREFERENCED_VARIABLE(pMem);
+    PORT_MEM_ATOMIC_DEC_U32(&pCounter->activeAllocs);
+#if PORT_MEM_TRACK_USE_FENCEPOSTS
+    PORT_MEM_ATOMIC_SUB_SIZE(&pCounter->activeSize,
+                             ((PORT_MEM_HEADER *)pMem-1)->fence.blockSize);
+#endif
 }
 
 #define PORT_MEM_COUNTER_INIT(pCounter)      _portMemCounterInit(pCounter)
@@ -264,7 +302,7 @@ _portMemListAdd
     PORT_MEM_LIST *pList = &pHead->list;
     pList->pNext = pList;
     pList->pPrev = pList;
-    if (!portAtomicCompareAndSwapSize(&pTracking->pFirstAlloc, pList, NULL))
+    if (!PORT_MEM_ATOMIC_CAS_SIZE(&pTracking->pFirstAlloc, pList, NULL))
     {
         PORT_LOCKED_LIST_LINK(pTracking->pFirstAlloc, pList, pTracking->listLock);
     }
@@ -279,11 +317,11 @@ _portMemListRemove
     PORT_MEM_HEADER *pHead = (PORT_MEM_HEADER*)pMem - 1;
     PORT_MEM_LIST *pList = &pHead->list;
 
-    if (!portAtomicCompareAndSwapSize(&pList->pNext, NULL, pList))
+    if (!PORT_MEM_ATOMIC_CAS_SIZE(&pList->pNext, NULL, pList))
     {
         PORT_LOCKED_LIST_UNLINK(pTracking->pFirstAlloc, pList, pTracking->listLock);
     }
-    portAtomicCompareAndSwapSize(&pTracking->pFirstAlloc, pList->pNext, pList);
+    PORT_MEM_ATOMIC_CAS_SIZE(&pTracking->pFirstAlloc, pList->pNext, pList);
 }
 
 static NV_INLINE PORT_MEM_HEADER *
@@ -378,7 +416,7 @@ typedef struct PORT_MEM_LOG_ENTRY
 #define PORT_MEM_LOG_ENTRIES 4096
 
 static void
-_portMemLogInit()
+_portMemLogInit(void)
 {
     NVLOG_BUFFER_HANDLE hBuffer;
     nvlogAllocBuffer(PORT_MEM_LOG_ENTRIES * sizeof(PORT_MEM_LOG_ENTRY),
@@ -387,7 +425,7 @@ _portMemLogInit()
 }
 
 static void
-_portMemLogDestroy()
+_portMemLogDestroy(void)
 {
     NVLOG_BUFFER_HANDLE hBuffer;
     nvlogGetBufferHandleFromTag(PORT_MEM_TRACK_LOG_TAG, &hBuffer);
@@ -439,7 +477,7 @@ static void *_portMemAllocatorAllocNonPagedWrapper(PORT_MEM_ALLOCATOR *pAlloc, N
 static void _portMemAllocatorFreeWrapper(PORT_MEM_ALLOCATOR *pAlloc, void *pMem);
 static void _portMemAllocatorReleaseWrapper(PORT_MEM_ALLOCATOR *pAlloc);
 
-static PORT_MEM_ALLOCATOR *_portMemAllocatorCreateOnExistingBlock(void *pAlloc, NvLength blockSizeBytes, void *pSpinlock);
+static PORT_MEM_ALLOCATOR *_portMemAllocatorCreateOnExistingBlock(void *pAlloc, NvLength blockSizeBytes, void *pSpinlock PORT_MEM_CALLERINFO_COMMA_TYPE_PARAM);
 static void    *_portMemAllocatorAllocExistingWrapper(PORT_MEM_ALLOCATOR *pAlloc, NvLength length);
 static void     _portMemAllocatorFreeExistingWrapper(PORT_MEM_ALLOCATOR *pAlloc, void *pMem);
 
@@ -485,8 +523,87 @@ static struct PORT_MEM_GLOBALS
     } alloc;
     NvU32 initCount;
     NvU32 totalAllocators;
+#if PORT_MEM_TRACK_USE_LIMIT
+    NvBool bLimitEnabled;
+    NvU64 limitPid[PORT_MEM_LIMIT_MAX_PIDS];
+    NvU64 counterPid[PORT_MEM_LIMIT_MAX_PIDS];
+#endif
 } portMemGlobals;
 
+//
+// Per-process heap limiting implementation
+//
+#if PORT_MEM_TRACK_USE_LIMIT
+static NV_INLINE void
+_portMemLimitInc(NvU32 pid, void *pMem, NvU64 size)
+{
+    PORT_MEM_HEADER *pMemHeader = PORT_MEM_SUB_HEADER_PTR(pMem);
+    pMemHeader->pid = pid;
+    if (portMemGlobals.bLimitEnabled)
+    {
+        if ((pid > 0) && (pid <= PORT_MEM_LIMIT_MAX_PIDS))
+        {
+            NvU32 pidIdx = pid - 1;
+            pMemHeader->blockSize = size;
+            PORT_MEM_ATOMIC_ADD_SIZE(&portMemGlobals.counterPid[pidIdx], size);
+        }
+    }
+}
+
+static NV_INLINE void
+_portMemLimitDec(void *pMem)
+{
+    if (portMemGlobals.bLimitEnabled)
+    {
+        PORT_MEM_HEADER *pMemHeader = PORT_MEM_SUB_HEADER_PTR(pMem);
+        NvU32 pid = pMemHeader->pid;
+
+        if ((pid > 0) && (pid <= PORT_MEM_LIMIT_MAX_PIDS))
+        {
+            NvU32 pidIdx = pid - 1;
+            if (portMemGlobals.counterPid[pidIdx] < pMemHeader->blockSize)
+            {
+                PORT_MEM_PRINT_ERROR("memory free error: counter underflow\n");
+                PORT_BREAKPOINT_CHECKED();
+            }
+            else
+            {
+                PORT_MEM_ATOMIC_SUB_SIZE(&portMemGlobals.counterPid[pidIdx], pMemHeader->blockSize);
+            }
+        }
+    }
+}
+
+static NV_INLINE NvBool
+_portMemLimitExceeded(NvU32 pid, NvU64 size)
+{
+    NvBool bExceeded = NV_FALSE;
+
+    if (portMemGlobals.bLimitEnabled)
+    {
+        if ((pid > 0) && (pid <= PORT_MEM_LIMIT_MAX_PIDS))
+        {
+            NvU32 pidIdx = pid - 1;
+            if ((size + portMemGlobals.counterPid[pidIdx]) > portMemGlobals.limitPid[pidIdx])
+            {
+                PORT_MEM_PRINT_ERROR("memory allocation denied; PID %d exceeded per-process heap limit of 0x%llx\n",
+                                      pid, portMemGlobals.limitPid[pidIdx]);
+                bExceeded = NV_TRUE;
+            }
+        }
+    }
+    return bExceeded;
+}
+
+#define PORT_MEM_LIMIT_INC(pid, pMem, size) _portMemLimitInc(pid, pMem, size)
+#define PORT_MEM_LIMIT_DEC(pMem)            _portMemLimitDec(pMem)
+#define PORT_MEM_LIMIT_EXCEEDED(pid, size)  _portMemLimitExceeded(pid, size)
+#else
+#define PORT_MEM_LIMIT_INC(pid, pMem, size)
+#define PORT_MEM_LIMIT_DEC(pMem)
+#define PORT_MEM_LIMIT_EXCEEDED(pid, size)  (NV_FALSE)
+#endif // PORT_MEM_TRACK_USE_LIMIT
+
 static NV_INLINE PORT_MEM_ALLOCATOR_TRACKING *
 _portMemGetTracking
 (
@@ -506,7 +623,7 @@ portMemInitialize(void)
 #if PORT_MEM_TRACK_USE_CALLERINFO
     PORT_MEM_CALLERINFO_TYPE_PARAM = PORT_MEM_CALLERINFO_MAKE;
 #endif
-    if (portAtomicIncrementU32(&portMemGlobals.initCount) != 1)
+    if (PORT_MEM_ATOMIC_INC_U32(&portMemGlobals.initCount) != 1)
         return;
 
     portMemGlobals.mainTracking.pAllocator = NULL;
@@ -516,6 +633,13 @@ portMemInitialize(void)
     PORT_MEM_LIST_INIT(&portMemGlobals.mainTracking);
     PORT_MEM_LOCK_INIT(portMemGlobals.trackingLock);
 
+#if PORT_MEM_TRACK_USE_LIMIT
+    // Initialize process heap limit to max int (i.e. no limit)
+    portMemGlobals.bLimitEnabled = NV_FALSE;
+    portMemSet(&portMemGlobals.limitPid, NV_U8_MAX, sizeof(portMemGlobals.limitPid));
+    portMemSet(&portMemGlobals.counterPid, 0, sizeof(portMemGlobals.counterPid));
+#endif
+
     portMemGlobals.alloc.paged._portAlloc      = _portMemAllocatorAllocPagedWrapper;
     portMemGlobals.alloc.nonPaged._portAlloc   = _portMemAllocatorAllocNonPagedWrapper;
     portMemGlobals.alloc.paged._portFree       = _portMemAllocatorFreeWrapper;
@@ -552,7 +676,7 @@ void
 portMemShutdown(NvBool bForceSilent)
 {
     PORT_UNREFERENCED_VARIABLE(bForceSilent);
-    if (portAtomicDecrementU32(&portMemGlobals.initCount) != 0)
+    if (PORT_MEM_ATOMIC_DEC_U32(&portMemGlobals.initCount) != 0)
         return;
 
 #if (PORT_MEM_TRACK_PRINT_LEVEL > PORT_MEM_TRACK_PRINT_LEVEL_SILENT)
@@ -560,7 +684,7 @@ portMemShutdown(NvBool bForceSilent)
     {
         portMemPrintTrackingInfo(NULL);
     }
-#endif 
+#endif
     PORT_MEM_LOG_DESTROY();
 
     if (PORT_MEM_TRACK_USE_FENCEPOSTS)
@@ -586,6 +710,9 @@ portMemAllocPaged
     PORT_MEM_CALLERINFO_COMMA_TYPE_PARAM
 )
 {
+#if defined(__COVERITY__)
+    return __coverity_alloc__(length);
+#endif
     PORT_MEM_ALLOCATOR *pAlloc = portMemAllocatorGetGlobalPaged();
     return _portMemAllocatorAlloc(pAlloc, length PORT_MEM_CALLERINFO_COMMA_PARAM);
 }
@@ -597,6 +724,9 @@ portMemAllocNonPaged
     PORT_MEM_CALLERINFO_COMMA_TYPE_PARAM
 )
 {
+#if defined(__COVERITY__)
+    return __coverity_alloc__(length);
+#endif
     PORT_MEM_ALLOCATOR *pAlloc = portMemAllocatorGetGlobalNonPaged();
     return _portMemAllocatorAlloc(pAlloc, length PORT_MEM_CALLERINFO_COMMA_PARAM);
 }
@@ -640,7 +770,7 @@ portMemAllocatorCreatePaged(PORT_MEM_CALLERINFO_TYPE_PARAM)
     portMemInitializeAllocatorTracking(pAllocator, &pAllocator->pImpl->tracking
                                        PORT_MEM_CALLERINFO_COMMA_PARAM);
 
-    PORT_MEM_PRINT_INFO("Acquired paged allocator %p", pAllocator);
+    PORT_MEM_PRINT_INFO("Acquired paged allocator %p ", pAllocator);
     PORT_MEM_PRINT_INFO(PORT_MEM_CALLERINFO_PRINT_ARGS(PORT_MEM_CALLERINFO_PARAM));
 
     return pAllocator;
@@ -663,7 +793,7 @@ portMemAllocatorCreateNonPaged(PORT_MEM_CALLERINFO_TYPE_PARAM)
     portMemInitializeAllocatorTracking(pAllocator, &pAllocator->pImpl->tracking
                                        PORT_MEM_CALLERINFO_COMMA_PARAM);
 
-    PORT_MEM_PRINT_INFO("Acquired nonpaged allocator %p", pAllocator);
+    PORT_MEM_PRINT_INFO("Acquired nonpaged allocator %p ", pAllocator);
     PORT_MEM_PRINT_INFO(PORT_MEM_CALLERINFO_PRINT_ARGS(PORT_MEM_CALLERINFO_PARAM));
     return pAllocator;
 }
@@ -678,7 +808,7 @@ portMemAllocatorCreateOnExistingBlock
 )
 {
     return _portMemAllocatorCreateOnExistingBlock(pPreallocatedBlock, blockSizeBytes,
-                                                  NULL);
+                                                  NULL PORT_MEM_CALLERINFO_COMMA_PARAM);
 }
 
 PORT_MEM_ALLOCATOR *
@@ -691,7 +821,7 @@ portMemExAllocatorCreateLockedOnExistingBlock
 )
 {
     return _portMemAllocatorCreateOnExistingBlock(pPreallocatedBlock, blockSizeBytes,
-                                                  pSpinlock);
+                                                  pSpinlock PORT_MEM_CALLERINFO_COMMA_PARAM);
 }
 
 void
@@ -747,9 +877,19 @@ portMemInitializeAllocatorTracking
     PORT_MEM_COUNTER_INIT(&pTracking->counter);
     PORT_MEM_LIST_INIT(pTracking);
     PORT_MEM_CALLERINFO_INIT_TRACKING(pTracking);
-    portAtomicIncrementU32(&portMemGlobals.totalAllocators);
+    PORT_MEM_ATOMIC_INC_U32(&portMemGlobals.totalAllocators);
 }
 
+#if PORT_MEM_TRACK_USE_LIMIT
+void
+portMemInitializeAllocatorTrackingLimit(NvU32 pid, NvU64 limit, NvBool bLimitEnabled)
+{
+    NvU32 pidIdx = pid - 1;
+    portMemGlobals.limitPid[pidIdx] = limit;
+    portMemGlobals.bLimitEnabled = bLimitEnabled;
+}
+#endif
+
 void *
 _portMemAllocatorAlloc
 (
@@ -758,12 +898,31 @@ _portMemAllocatorAlloc
     PORT_MEM_CALLERINFO_COMMA_TYPE_PARAM
 )
 {
+#if PORT_MEM_TRACK_USE_LIMIT
+    NvU32 pid = 0;
+#endif
     void *pMem = NULL;
     if (pAlloc == NULL)
     {
         PORT_BREAKPOINT_CHECKED();
         return NULL;
     }
+
+#if PORT_MEM_TRACK_USE_LIMIT
+    if (portMemGlobals.bLimitEnabled)
+    {
+        if (osGetCurrentProcessGfid(&pid) != NV_OK)
+        {
+            PORT_BREAKPOINT_CHECKED();
+            return NULL;
+        }
+    }
+#endif
+
+    // Check if per-process memory limit will be exhausted by this allocation
+    if (PORT_MEM_LIMIT_EXCEEDED(pid, length))
+        return NULL;
+
     if (length > 0)
     {
         NvLength paddedLength;
@@ -788,6 +947,7 @@ _portMemAllocatorAlloc
         pMem = PORT_MEM_ADD_HEADER_PTR(pMem);
         _portMemTrackAlloc(_portMemGetTracking(pAlloc), pMem, length
                            PORT_MEM_CALLERINFO_COMMA_PARAM);
+        PORT_MEM_LIMIT_INC(pid, pMem, length);
     }
     return pMem;
 }
@@ -805,6 +965,7 @@ _portMemAllocatorFree
     }
     if (pMem != NULL)
     {
+        PORT_MEM_LIMIT_DEC(pMem);
         _portMemTrackFree(_portMemGetTracking(pAlloc), pMem);
         pMem = PORT_MEM_SUB_HEADER_PTR(pMem);
         pAlloc->_portFree(pAlloc, pMem);
@@ -1029,7 +1190,7 @@ portMemExTrackingGetNext
 
     pHead = _portMemListGetHeader(pList);
 
-    // Advance itertator
+    // Advance iterator
     if (pList->pNext == pTracking->pFirstAlloc)
         *pIterator = NULL;
     else
@@ -1067,7 +1228,7 @@ _portMemTrackingRelease
 
     PORT_LOCKED_LIST_UNLINK(&portMemGlobals.mainTracking, pTracking, portMemGlobals.trackingLock);
     PORT_MEM_LIST_DESTROY(pTracking);
-    portAtomicDecrementU32(&portMemGlobals.totalAllocators);
+    PORT_MEM_ATOMIC_DEC_U32(&portMemGlobals.totalAllocators);
 }
 
 static void
@@ -1174,6 +1335,7 @@ _portMemAllocatorCreateOnExistingBlock
     void *pPreallocatedBlock,
     NvLength blockSizeBytes,
     void *pSpinlock
+    PORT_MEM_CALLERINFO_COMMA_TYPE_PARAM
 )
 {
     PORT_MEM_ALLOCATOR *pAllocator = (PORT_MEM_ALLOCATOR *)pPreallocatedBlock;
@@ -1225,7 +1387,7 @@ _portMemAllocatorCreateOnExistingBlock
     }
     portMemSet(pBitVector->bits, 0, bitVectorSize);
 
-    PORT_MEM_PRINT_INFO("Acquired preallocated block allocator %p (%llu bytes)", pAllocator, (NvU64)blockSizeBytes);
+    PORT_MEM_PRINT_INFO("Acquired preallocated block allocator %p (%llu bytes) ", pAllocator, (NvU64)blockSizeBytes);
     PORT_MEM_PRINT_INFO(PORT_MEM_CALLERINFO_PRINT_ARGS(PORT_MEM_CALLERINFO_PARAM));
     return pAllocator;
 }
